<?php
namespace Teknisa\PLA\Modules\Prato\Controllers;

use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\DataSet as DataSet_Source;
use DTO\Request\DataSet;
use \Zeedhi\Framework\DTO;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;

class Prato extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $parameterBagColumns = array();
    protected $dataSourceManager;

    protected $pratoService;
    protected $environment;
    protected $paramPrato;
    protected $paramgeral;
    protected $newCode;
    protected $connection;
    protected $authController;

    public function __construct(
        $crudService, $parameterBag, $dataSourceManager,
        \Teknisa\PLA\Modules\Prato\Services\PratoService $pratoService,
        \Teknisa\GER\Utilities\Environment $environment,
        \Teknisa\GER\Services\ParamgeralService $paramgeral,
        \Teknisa\GER\Services\NewCode $newCode,
        $util, $connection, $authController
        
    ) {
        parent::__construct($crudService, $parameterBag);
        $this->dataSourceManager = $dataSourceManager;
        $this->pratoService = $pratoService;
        $this->environment = $environment;
        $this->paramgeral = $paramgeral;
        $this->newCode = $newCode;
        $this->util = $util;
        $this->connection = $connection;
        $this->authController = $authController;
    }
    protected $keysMapping = array(
            'QTPROD' => array(
                'S' => 'QTPRODRECSER',
                'L' => 'QTPRODRECLOC',
                'R' => 'QTPRODRECREGIAO',
                'Q' => 'QTPRODRECPADQ',
                'P' => 'QTPRODPRAT'
            ),
            'VRPEFATCOC' => array(
                'S' => 'VRPEFATCOCSE',
                'L' => 'VRPEFATCOCLO',
                'R' => 'VRPEFATCOCREGIAO',
                'Q' => 'VRPEFATCOCPADQ',
                'P' => 'VRPEFATCOCPR'
            ),
            'VRPEFATCOR' => array(
                'S' => 'VRPEFATCORSE',
                'L' => 'VRPEFATCORLO',
                'R' => 'VRPEFATCORREGIAO',
                'Q' => 'VRPEFATCORPADQ',
                'P' => 'VRPEFATCORPR'
            ),
            'QTBRUTA' => array(
                'S' => 'QTBRUTASE',
                'L' => 'QTBRUTALO',
                'R' => '',
                'Q' => '',
                'P' => 'QTBRUTAPR'
            ),
            'QTUNIDEST' => array(
                'S' => 'QTUNIDESTSE',
                'L' => 'QTUNIDESTLO',
                'R' => '',
                'Q' => '',
                'P' => 'QTUNIDESTPR'
            ),
            'VRPEFATNUT' => array(
                'S' => 'VRPEFATNUTSE',
                'L' => 'VRPEFATNUTLO',
                'R' => '',
                'Q' => '',
                'P' => 'VRPEFATNUTPR'
            ),
            'QTCALCNUTR' => array(
                'S' => 'QTCALCNUTRSE',
                'L' => 'QTCALCNUTRLO',
                'R' => '',
                'Q' => '',
                'P' => 'QTCALCNUTRPR'
            ),
            'QTBRUTAUNR' => array(
                'S' => 'QTBRUTASEUNR',
                'L' => 'QTBRUTALOUNR',
                'R' => 'QTBRUTAREGIAOUNR',
                'Q' => 'QTBRUTAPADQUNR',
                'P' => 'QTBRUTAPRUNR'
            ),
            'HRTEMPPREP' => array(
                'S' => 'HRTEMPPREPSE',
                'L' => 'HRTEMPPREPLO',
                'R' => '',
                'Q' => '',
                'P' => 'HRTEMPPREPPR'
            ),
            'IDPRODBASE' => array(
                'S' => 'IDPRODBASESE',
                'L' => 'IDPRODBASELO',
                'R' => 'IDPRODBASEREGIAO',
                'Q' => 'IDPRODBASEPR',
                'P' => 'IDPRODBASEPR'
            ),
            'IDQTDEFIX' => array(
                'S' => 'IDQTDEFIXSE',
                'L' => 'IDQTDEFIXLO',
                'R' => 'IDQTDEFIXREGIAO',
                'Q' => 'IDQTDEFIXPADQ',
                'P' => 'IDQTDEFIXPR'
            ),
            'QTCOMPLPROD' => array(
                'S' => 'QTCOMPLPRODSERV',
                'L' => 'QTCOMPLPRODFILI',
                'R' => '',
                'Q' => '',
                'P' => 'QTCOMPLPRODPADR'
            ),
            'IDTPREQPROD' => array(
                'S' => 'IDTPREQPRODSERV',
                'L' => 'IDTPREQPRODFILI',
                'R' => '',
                'Q' => '',
                'P' => 'IDTPREQPRODPADR'
            ),
            'QTLIMSEG' => array(
                'S' => 'QTLIMSEGSERV',
                'L' => 'QTLIMSEGFILI',
                'R' => '',
                'Q' => '',
                'P' => 'QTLIMSEGPADR',
            ),
            'CODIGO' => array(
                'S' => '',
                'L' => '',
                'R' => 'NRRECEREGIAO',
                'Q' => 'NRRECEPADQUALI',
                'P' => '',
            )
        );

    // INICIO PRATO
    public function carregaPratos(DTO\Request\Filter $request,
        DTO\Response $response)
    {
        $conditions = $request->getFilterCriteria()->getConditions();
        
        $filial = $conditions[0]['value'];
        if($filial != "" || $filial != null) {
            if (array_key_exists(1, $conditions)) {
                if ($conditions[1]['operator'] == 'LIKE_I'){
                    //se for pesquisar pelos filhos retorna
                    $return = false;
                    foreach($conditions as $cond){
                        if($cond['columnName'] == "CDPRATONIVANTE") $return =  true;
                    }
                    if($return) return true;
                    
                    $where = [];
                    for($i = 1;$i < sizeof($conditions); $i++){
                        if($conditions[$i]['columnName'] == "IDTIPO"){
                            array_push($where,"UPPER(". "CDFILAUXPRAT" . ") like UPPER('" . $conditions[$i]['value'] ."')");
                        } else {
                            array_push($where,"UPPER(". $conditions[$i]['columnName'] . ") like UPPER('" . $conditions[$i]['value'] ."')");
                        }
                    }
                    $where = implode(" AND ",$where);
                    
                    $pratos = $this->pratoService->retornaPratosEpais($filial, $where);
                    $response->addDataset(new DataSet_Source('prato_unidade', $pratos));
                    return true;
                    
                }else if (strpos($conditions[1]['operator'], 'LIKE_ALL') !== false) {
                    //filtro da árvore - CDPRATO|NMPRATO|IDTIPO
                    $value = $conditions[1]['value'];
                    if(is_array($conditions[1]['value'])){
                        $value = $conditions[1]['value'][0]['_ALL'];
                    }
                    $where = "UPPER (CDPRATO||NMPRATO||DECODE(CDFILAUXPRAT, 'P', 'PADRAO', 'LOCAL')) like UPPER('" . $value."')";
                    $pratos = $this->pratoService->retornaPratosFilterAll($filial, $where);
                    $response->addDataset(new DataSet_Source('prato_unidade', $pratos));
    
                    return true;
                } else {
                    //carrega registros filhos - CDPRATONIVANTE
                    $cdpratoPai = $conditions[1]['value'];
                    $where = "CDPRATONIVANTE = '" . str_replace('.', '', $cdpratoPai) . "'";
                }
            } else {
                //carrega pais de todos - CDPRATONIVANTE NULL
                $where = "CDPRATONIVANTE IS NULL";
            }
            $pratos = $this->pratoService->retornaPratos($filial, $where);
            $response->addDataset(new DataSet_Source('prato_unidade', $pratos));    
        }
    }


    public function retornaCodigo(DTO\Request\Row $request,DTO\Response $response){
        
        $row = $request->getRow();
        $pratoPai = $row['PRATOPAI'];
        $filAuxPai = $row['CDFILAUXPRAT'];
        $filial = $row['FILIAL'];
        //$nivel = $row['NIVEL'];
        $ultimoFilho = null;
        $maxCdPrato = 0;
        $padRightParentPlate = str_pad($pratoPai, 10, "X", STR_PAD_RIGHT);
        $filhos = $this->pratoService->retornaFilhosfiliauxpratMaisFilial($pratoPai,$filAuxPai,$filial);
        /* VALIDACAO PARA SUGERIR CODIGO EM 'FALHAS' SEQUENCIAIS
                EX:
                    01
                    03 -> o codigo sugerido seria 02*/
        if ($filhos) {
            //$aux = $primeiroFilho = $filhos[0]->getCdprato(); 
            $totalFilhos = count($filhos) - 1;
            $ultimoFilho = $filhos[$totalFilhos]->getCdprato(); 
            /*
            $tamCaractFilho = strlen($primeiroFilho); 
            $provaveisFilhos = array();
            $filhosExistentes = array();
            $filhosExistentesNaoRepetido = array();

            while ($aux <= $ultimoFilho) {
                $provaveisFilhos[] = str_pad($primeiroFilho++, $tamCaractFilho, '0', STR_PAD_LEFT);
                $aux++;
            }

            for($i = 0; $i<count($filhos); $i++){
                $filhosExistentes[$i] =  $filhos[$i]->getCdprato();
                if($i>0){
                    if($filhosExistentes[$i] <> $filhosExistentes[$i-1]){
                        array_push($filhosExistentesNaoRepetido,$filhosExistentes[$i]); 
                    }
                }
                else{
                    array_push($filhosExistentesNaoRepetido,$filhosExistentes[$i]);
                }
            }

            foreach ($provaveisFilhos as $chave => $filho) {
                if ($filho <> $filhosExistentesNaoRepetido[$chave]){
                    $maxCdPrato = $filho;
                    break;
                } else {
                    $maxCdPrato = ++$filho;
                }
            }*/
        }
        
        $mask = $this->pratoService->getCdPictPrat();
        if($ultimoFilho == null){
            $formatCodeWithX = $this->pratoService->getFormatCodePlate($padRightParentPlate, $mask);
            $matches = array();
            preg_match("/X+/", $formatCodeWithX, $matches);
            $sizeToComplete = strlen($matches[0]);
            $maxCdPrato = str_pad($pratoPai, $sizeToComplete+strlen($pratoPai), "0", STR_PAD_RIGHT);
        } else {
            $formatCodeLastPlate = $this->pratoService->getFormatCodePlate($ultimoFilho, $mask);
            $lastPart = explode(".", $formatCodeLastPlate);
            $lastPart = $lastPart[count($lastPart)-1];
            if(preg_match("/^9+$/", $lastPart) == 1){
                $maxCdPrato = $ultimoFilho;
            } else {
                $maxCdPrato = $ultimoFilho+1;
            }
        }
        $response->addDataset(new DataSet_Source('CODIGO', array($maxCdPrato)));
    }


    public function verificaExclusaoPrato(DTO\Request\Row $request, DTO\Response $response){

        $row = $request->getRow();
        $pratoPai = $row['PRATOPAI'];
        $filAuxPai = $row['CDFILAUXPRAT'];

        $filhos = $this->pratoService->retornaFilhos($pratoPai, $filAuxPai);
        if ($filhos) {
            $response->addDataset(new DataSet_Source('FILHOS', array(true)));
        } else {
            $response->addDataset(new DataSet_Source('FILHOS', array(false)));
        }
    }


    public function exclusaoCascadePrato(
        DTO\Request\Row $request,
        DTO\Response $response
    ) {
        try {
            $row = $request->getRow();
            $prato = $row['CDPRATO'];
            $filAux = $row['CDFILAUXPRAT'];
            $nivel = $row['NIVEL'];
            
            $this->pratoService->iniciaTransacao();

            $this->pratoService->excluiContainerPrato($row['CDPRATO'], $row['CDFILAUXPRAT']);
            $conn = $this->pratoService->getConnection();
            $exclusao = new StoredProcedure($conn, 'DEL_CASCADE_PRATO');

            $exclusao->addParam(new Param('P_CDFILAUXPRAT', Param::PARAM_INPUT, $filAux, Param::PARAM_TYPE_STR));
            $exclusao->addParam(new Param('P_CDPRATO', Param::PARAM_INPUT, $prato, Param::PARAM_TYPE_STR));
            $exclusao->execute();
            
            if($nivel == 5)
                $this->pratoService->excluiPrato($prato, $filAux);
            else
                $this->pratoService->excluiNivelVazio($prato, $filAux);

            $response->addDataset(new DataSet_Source('EXCLUSAO', array(true)));

            $this->pratoService->commit();
        } catch (\Exception $e) {
            $this->pratoService->rollback();
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }


    public function retornaPrato(DTO\Request\Row $request,
        DTO\Response $response){

        $row = $request->getRow();
        $prato = $row['CDPRATO'];
        $cdfilial = $row['CDFILIAL'] == 'T' ? 'P' : $row['CDFILIAL']; // caso CDFILIAL passe como 'P', a validação da libs reclama de unidade inválida 
        $prato = $this->pratoService->retornaPrato($prato, $cdfilial);
        $conditions = array(
            array(
                "columnName" => "NRORG",
                "operator" => "=",
                "value" => $this->environment->getNrOrgTrab()
            ),
            array(
                "columnName" => "CDPRATO",
                "operator" => "=",
                "value" => $row['CDPRATO']
            )
        );
        $dataSet = $this->crudService->getFilteredDataset("prato_add", $conditions);
        
        if ($prato) {
            $response->addDataset(
                new DataSet_Source('PRATO', array(
                    'CDPRATO' => $prato[0]['CDPRATO'], 
                    'CDFILAUXPRAT' => $prato[0]['CDFILAUXPRAT'],
                    'PRATOOUTRAUNIDADE' => !empty($dataSet->getRows())
                    )
                )
            );
        } else {
            $response->addDataset(
                new DataSet_Source('PRATO', array(
                    'CDPRATO' => false,
                    'PRATOOUTRAUNIDADE' => !empty($dataSet->getRows())
                    )
                )
            );
        }
    }


    public function retornaFatorConversaoPrato(
        DTO\Request\Row $request,
        DTO\Response $response
    ){
        $row = $request->getRow();

        $produtosRec = $row['PRODUTOS'];
        $sgunidadeCompar = $row['SGUNIDADEPRAT'];
        $qtAtiva = $row['QTATIVA'];
        $nrorg = $this->environment->getNrOrgTrab();
        $qtd = 0;

        foreach ($produtosRec as $produto) {
            $valorQtd = $produto[$qtAtiva];
            $sgUnidProd = $produto['SGUNIDADE'] ? $produto['SGUNIDADE'] : $produto['SGUNIDADEPROD'];
            if ($sgUnidProd != $sgunidadeCompar) {
                $fatorConv = $this->pratoService->retornaFatorConversaoPrato(
                    $produto['CDPRODUTO'], $sgUnidProd,
                    $sgunidadeCompar, $nrorg
                );
                if(!empty($fatorConv)) {
                    if ($fatorConv[0]['VRFATOCONVES'] === 0) {
                        $qtd = $qtd + $valorQtd;
                    } else {
                        $invertValor = ($fatorConv[0]['ORDEM'] === 3);
        
                        if ($fatorConv[0]['IDTIPOPE'] === '*') {
                            if ($invertValor === true) {
                                $qtd = $qtd + ($valorQtd / $fatorConv[0]['VRFATOCONVES']);
                            } else {
                                $qtd = $qtd + ($valorQtd * $fatorConv[0]['VRFATOCONVES']);
                            }
                        } else {
                            if ($invertValor === true) {
                                $qtd = $qtd + ($valorQtd * $fatorConv[0]['VRFATOCONVES']);
                            } else {
                                $qtd = $qtd + ($valorQtd / $fatorConv[0]['VRFATOCONVES']);
                            }
                        }
                    }
                }
            } else {
                $qtd = $qtd + $valorQtd;
            }
        }
        $response->addDataset(new DataSet_Source('QTPESSOA', array($qtd)));
    }

    public function retornaPratoSemConsumo(
        DTO\Request\Filter $request,
        DTO\Response $response
    ){
        $this->dataSourceName = "prato_sem_percapita_fixo";
        $this->parameterBagColumns = array('NRORG', 'CDFILIAL');

        $this->find($request, $response);
    }
    // FIM PRATO


    // INICIO EMBALAGEM
    public function atualizaEmbalagemPrincipal(
        DTO\Request\Row $request,
        DTO\Response $response)
    {
        $dataSourceName = "pratocontainer";
        $rows = $this->crudService->getRowsToSave($request, $dataSourceName);
        $this->crudService->save($dataSourceName, $rows);

        return true;
    }
    // FIM EMBALAGEM


    // INICIO MODO DE PREPARO
    public function retornaModoPreparo(
        DTO\Request\Filter $request,
        DTO\Response $response
    ){
        $conditions = $request->getFilterCriteria()->getConditions();
        $receita = null;
        $cdservfili = false;
        $cdfilial = false;
        $cdprato = '';
        $cdfilauxprat = '';
        $sql = '';
        
        foreach($conditions as $currentCondition){
            $cdprato      = $currentCondition['columnName'] === 'CDPRATO'      ? $currentCondition['value'] : $cdprato;
            $cdfilial     = $currentCondition['columnName'] === 'CDFILIAL'     ? $currentCondition['value'] : $cdfilial;
            $cdservfili   = $currentCondition['columnName'] === 'CDSERVFILI'   ? $currentCondition['value'] : $cdservfili;
            $cdfilauxprat = $currentCondition['columnName'] === 'CDFILAUXPRAT' ? $currentCondition['value'] : $cdfilauxprat;
            if($currentCondition['operator'] === 'LIKE_I'){
                $sql .= "AND UPPER (".$currentCondition['columnName'].") LIKE UPPER('".$currentCondition['value']."')";
            }
            if($currentCondition['operator'] === 'MAPPED_LIKE_ALL'){
                $sql = "AND UPPER (".str_replace(array('|', '#'), '||', $currentCondition['columnName']).") LIKE UPPER('".$currentCondition['value'][0]['_ALL']."')";
            }
        }
        
        if ($conditions) {
            $receita = "P";

            if ($cdfilial !== false) {
                $receita = "L";
            }

            if ($cdservfili !== false) {
                $receita = "S";
            }
        }
        
        $nameDataSource = 'prepprapad_format';
        switch ($receita) {
        case "P":
            $modos = $this->pratoService->retornaPreparoPadrao($cdprato, $cdfilauxprat, $sql);
            break;
        case "L":
            $nameDataSource = 'prepprafil_format';
            $modos = $this->pratoService->retornaPreparoLocal($cdprato, $cdfilauxprat, $cdfilial, $sql);
            break;
        case "S":
            $nameDataSource = 'preppraser_format';
            $modos = $this->pratoService->retornaPreparoServico($cdprato, $cdfilauxprat, $cdfilial, $cdservfili, $sql);
            break;
        default:
            $modos = array();
            break;
        }
        $response->addDataset(new DataSet_Source($nameDataSource, $modos));
    }
    
    public function insereAtualizaDadosReceitaPratoCustom(
        DTO\Request\Row $request,
        DTO\Response $response
    ){
        try {
            $row = $request->getRow();

            if($row['TIPO'] == 'Q'){
                $dataSourceName = 'pratpadquali';
                
                if($row['__is_new'] || !isset($row['NRPRATPADQUALI'])){
                    $row['__is_new'] = true;
                    $row['NRPRATPADQUALI'] = $this->newCode->getCodeByTableName('PRATPADQUALI', $this->environment->getNrOrgTrab());
                }
                $row['NRPRATPADQUALI'] = intval($row['NRPRATPADQUALI']);
                $rows = $this->crudService->getRowsToSave($request, $dataSourceName);
                $this->crudService->save($dataSourceName, $rows);
                $response->addDataset(new DataSet_Source('CALLBACK', array($row['NRPRATPADQUALI'])));
                
            }else if($row['TIPO'] == 'R'){
                $dataSourceName = 'pratregiao';
                if($row['__is_new'] || !isset($row['NRPRATREGIAO'])){
                    $row['__is_new'] = true;
                    $row['NRPRATREGIAO'] = $this->newCode->getCodeByTableName('PRATREGIAO', $this->environment->getNrOrgTrab());
                }
                $rows = $this->crudService->getRowsToSave($request, $dataSourceName);
                $this->crudService->save($dataSourceName, $rows);
                $response->addDataset(new DataSet_Source('CALLBACK', array($row['NRPRATREGIAO'])));
            }
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }

    public function insereAtualizaModoPreparo(
        DTO\Request\Row $request,
        DTO\Response $response
    ){
        try {
            $row = $request->getRow();

            $prato      = $row['CDPRATO'];
            $filAux     = $row['CDFILAUXPRAT'];
            $status     = $row['STATUS'];
            $texto      = $row['TEXTO'];
            $reponsavel = $row['RESPONSAVEL'];
            $tipo       = $row['TIPO'];
            $codeReturn;

            $data = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
            $nrorg = $this->environment->getNrOrgTrab();
            $operador = $this->environment->getCdOperador();

             switch ($tipo) {
                 case 'S':
                     $filial = $row['CDFILIAL'];
                     $servico = $row['CDSERVFILI'];
    
                     $modoExistentes = $this->pratoService->retornaPreparoServico($prato, $filAux, $filial, $servico);
                     if ($modoExistentes) {
                         $codigo = $modoExistentes[0]['NRPREPSERV'] + 1;
                         $trat = $this->trataCodigoPreparo($codigo);
                         if (!$trat) {
                             $response->addDataset(new DataSet_Source('CALLBACK', array('ERRO_CODIGO')));
                             return false;
                         }
                         $codigo = str_pad($codigo, 2, "0", STR_PAD_LEFT);
    
                         if ($status == '1')
                             $this->pratoService->atualizaStatusPreparoServico($filAux, $prato, $filial, $servico);
                     } else {
                         $codigo = '01';
                     }
    
                     if (!$row["CODIGO"]) {
                         $this->pratoService->inserePreparoServico(
                             $filAux, $prato, $filial, $servico, $codigo, $status,
                             $texto,
                             $reponsavel, $data, $nrorg, $operador
                         );
                     } else {
                         $this->pratoService->atualizaPreparoServico(
                             $filAux, $prato, $filial, $servico, $row['CODIGO'], $status,
                             $texto, $reponsavel, $data, $nrorg, $operador);
                     }
                     $codeReturn = true;
                 break;
                 case 'L':
                     $filial = $row['CDFILIAL'];
    
                     $modoExistentes = $this->pratoService->retornaPreparoLocal($prato, $filAux, $filial);
                     if ($modoExistentes) {
                         $codigo = $modoExistentes[0]['NRPREPFILI'] + 1;
                         $trat = $this->trataCodigoPreparo($codigo);
                         if (!$trat) {
                             $response->addDataset(new DataSet_Source('CALLBACK', array('ERRO_CODIGO')));
                             return false;
                         }
                         $codigo = str_pad($codigo, 2, "0", STR_PAD_LEFT);
    
                         if ($status == '1')
                             $this->pratoService->atualizaStatusPreparoLocal($filAux, $prato, $filial);
                     } else {
                         $codigo = '01';
                     }
    
                     if (!$row["CODIGO"]) {
                         $this->pratoService->inserePreparoLocal(
                             $filAux, $prato, $filial, $codigo, $status, $texto,
                             $reponsavel, $data, $nrorg, $operador
                         );
                     } else {
                         $this->pratoService->atualizaPreparoLocal(
                             $filAux, $prato, $filial, $row['CODIGO'], $status,
                             $texto, $reponsavel, $data, $nrorg, $operador);
                     }
                     $codeReturn = true;
                 break;
                 case 'P':
                     $modoExistentes = $this->pratoService->retornaPreparoPadrao($prato, $filAux);
                     if ($modoExistentes) {
                         $codigo = $modoExistentes[0]['NRPREPPRAT'] + 1;
                         $trat = $this->trataCodigoPreparo($codigo);
                         if (!$trat) {
                             $response->addDataset(new DataSet_Source('CALLBACK', array('ERRO_CODIGO')));
                             return false;
                         }
                         $codigo = str_pad($codigo, 2, "0", STR_PAD_LEFT);
    
                         if ($status == '1')
                             $this->pratoService->atualizaStatusPreparoPadrao($filAux, $prato);
                     } else {
                         $codigo = '01';
                     }
    
                     if (!$row["CODIGO"]) {
                         $this->pratoService->inserePreparoPadrao(
                             $filAux, $prato, $codigo, $status, $texto,
                             $reponsavel, $data, $nrorg, $operador
                         );
                    } else {
                        $this->pratoService->atualizaPreparoPadrao(
                            $filAux, $prato, $row['CODIGO'], $status,
                            $texto, $reponsavel, $data, $nrorg, $operador);
                    }
                    $codeReturn = true;
                break;
                case 'Q':
                    $modoExistentes = $this->pratoService->retornaPreparoPadraoQualidade($prato, $filAux);
                     if ($modoExistentes) {
                         $codigo = $modoExistentes[0]['NRPREPPADQ'] + 1;
                         $trat = $this->trataCodigoPreparo($codigo);
                         if (!$trat) {
                             $response->addDataset(new DataSet_Source('CALLBACK', array('ERRO_CODIGO')));
                             return false;
                         }
                         $codigo = str_pad($codigo, 2, "0", STR_PAD_LEFT);
    
                     } else {
                         $codigo = '01';
                     }
                     
                     $nrpad = $row['NRPAD'];
                     $id = $row['UNIQID'];
                     if (!$row["CODIGO"]) {
                         $newCode = $this->newCode->getCodeByTableName('PREPPRAPADQUALI', $this->environment->getNrOrgTrab());
                         $this->pratoService->inserePreparoPadraoQualidade(
                             $filAux, $prato, $codigo, $status, $texto,
                             $reponsavel, $data, $nrorg, $operador, $newCode, $nrpad
                         );
                         $codeReturn = $newCode;
                    } else {
                        $this->pratoService->atualizaPreparoPadraoQualidade(
                            $status, $texto, $reponsavel, $data, $nrorg, 
                            $operador, $id, $codigo
                        );
                        $codeReturn = true;
                    }
                break;
                case 'R':
                    $modoExistentes = $this->pratoService->retornaPreparoRegiao($prato, $filAux);
                     if ($modoExistentes) {
                         $codigo = $modoExistentes[0]['NRPREPREGIAO'] + 1;
                         $trat = $this->trataCodigoPreparo($codigo);
                         if (!$trat) {
                             $response->addDataset(new DataSet_Source('CALLBACK', array('ERRO_CODIGO')));
                             return false;
                         }
                         $codigo = str_pad($codigo, 2, "0", STR_PAD_LEFT);
    
                     } else {
                         $codigo = '01';
                     }
                     $regiao = $row['CDREGIAO'];
                     $nrpad = $row['NRPAD'];
                     $id = $row['UNIQID'];
                     if (!$row["CODIGO"]) {
                         $newCode = $this->newCode->getCodeByTableName('PREPPRAREGIAO', $this->environment->getNrOrgTrab());
                         $this->pratoService->inserePreparoRegiao(
                             $filAux, $prato, $codigo, $status, $texto,
                             $reponsavel, $data, $nrorg, $operador, $newCode, $regiao, $nrpad
                         );
                         $codeReturn = $newCode;
                    } else {
                        $this->pratoService->atualizaPreparoRegiao(
                            $status, $texto, $reponsavel, $data, $nrorg, 
                            $operador, $regiao, $id, $codigo
                        );
                        $codeReturn = true;
                    }
                break;
             }

            $response->addDataset(new DataSet_Source('CALLBACK', array($codeReturn)));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }

    private function trataCodigoPreparo($codigo){
        $tam = strlen($codigo);
        if ($tam > 2) {
            return false;
        } else {
            return true;
        }
    }
    // FIM MODO DE PREPARO


    // INICIO RECEITA
    public function retornaFatorConversaoProduto(
        DTO\Request\Row $request,
        DTO\Response $response
    ){
        $row = $request->getRow();
        $produto = $row['CDPRODUTO'];
        $sgunidade = $row['SGUNIDADE'];
        $nrorg = $this->environment->getNrOrgTrab();

        $unidades = $this->pratoService->retornaFatorConversao($produto, $sgunidade, $nrorg);
        $response->addDataset(new DataSet_Source('UNIDADES', array($unidades)));
    }


    public function retornaReceita(
        DTO\Request\Row $request,
        DTO\Response $response
    ){
        $row = $request->getRow();

        $cdprato = $row['CDPRATO'];
        $cdFilAuxPrat = $row['CDFILAUXPRAT'];

        try {
            $receita = $this->pratoService->retornaReceitaPrato($cdprato, $cdFilAuxPrat);

            if ($receita)
                $response->addDataset(new DataSet_Source('CALLBACK', array(true)));
            else
                $response->addDataset(new DataSet_Source('CALLBACK', array(false)));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }

    }


    public function salvaReceita(
        DTO\Request\Row $request,
        DTO\Response $response
    ){
        $row = $request->getRow();
        $this->pratoService->iniciaTransacao();

        // Alteração proporcional
        if (array_key_exists("ALTPROP", $row) && $row['ALTPROP'] === 'S' && array_key_exists("PERCENTUAL", $row)) {
            
            $this->alteracaoProporcional($request, $response);
        }
        try {
            switch($row['TIPORECEITA']){
                case 'S':
                    $this->dataSourceName = 'receserv';
                break;
                case 'L':
                    $this->dataSourceName = 'recefili';
                break;
                case 'P':
                    $this->dataSourceName = 'recepadr';
                break;
                case 'Q':
                    $this->dataSourceName = 'recepadquali';
                    if($row['__is_new']){
                        $row['NRRECEPADQUALI'] = $this->newCode->getCodeByTableName('RECEPADQUALI', $this->environment->getNrOrgTrab());
                    }
                break;
                case 'R':
                    $this->dataSourceName = 'receregiao';
                    if($row['__is_new']){
                        $row['NRRECEREGIAO'] = $this->newCode->getCodeByTableName('RECEREGIAO', $this->environment->getNrOrgTrab());
                    }
                break;
            }

            if (array_key_exists("CDPRODUTO_AUX", $row)) { // atualização
                if ($row['CDPRODUTO'] <> $row['CDPRODUTO_AUX']) {
                    $row['__is_new'] = false;
                    $produtoNovo = $row['CDPRODUTO'];
                    $row['CDPRODUTO'] = $row['CDPRODUTO_AUX'];

                    $this->delete($request, $response);
                    if ($response->getError()) {
                        throw new \Exception($response->getError()->getMessage());
                    }
                    $this->pratoService->flush();

                    $row['CDPRODUTO'] = $produtoNovo;
                    $row['__is_new'] = true;
                }
            }
        
            $conditions = array(
                array(
                    "columnName" => "CDPRATO",
                    "operator" => "=",
                    "value" => $row['CDPRATO']
                ),
                array(
                    "columnName" => "CDFILAUXPRAT",
                    "operator" => "=",
                    "value" => $row['CDFILAUXPRAT']
                ),
                array(
                    "columnName" => "CDPRODUTO",
                    "operator" => "=",
                    "value" => $row['CDPRODUTO']
                )
            );
            if($row['TIPORECEITA'] == 'R' || $row['TIPORECEITA'] == 'Q'){
                if($row['TIPORECEITA'] == 'R'){
                    array_push($conditions, 
                        array(
                            "columnName" => "CDREGIAO",
                            "operator" => "=",
                            "value" => $row['CDREGIAO']
                        )
                    );
                }
                array_push($conditions, 
                    array(
                        "columnName" => "NRPADRAOQUALIDADE",
                        "operator" => "=",
                        "value" => $row['NRPADRAOQUALIDADE']
                    )
                );
            }else if($row['TIPORECEITA'] == 'S' || $row['TIPORECEITA'] == 'L'){
                if($row['TIPORECEITA'] == 'S'){
                    array_push($conditions, 
                        array(
                            "columnName" => "CDSERVFILI",
                            "operator" => "=",
                            "value" => $row['CDSERVFILI']
                        )
                    );
                }
                array_push($conditions, 
                    array(
                        "columnName" => "CDFILIAL",
                        "operator" => "=",
                        "value" => $row['CDFILIAL']
                    )
                );
            }
            
            $filterCriteria = new \Zeedhi\Framework\DataSource\FilterCriteria($this->dataSourceName, $conditions);
            $dataSet = $this->dataSourceManager->findBy($filterCriteria);
            $receita = $dataSet->getRows();
            if($receita && $row['__is_new']){
                $response->addMessage(new DTO\Response\Message('Já Cadastrado!'));
            }else{
                $this->save($request, $response);
                if ($response->getError()) {
                    throw new \Exception($response->getError()->getMessage());
                }
                $this->pratoService->commit();
            }
        } catch (\Exception $e) {
            $this->pratoService->rollback();
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }


    public function alteracaoProporcional(DTO\Request\Row $request, DTO\Response $response){
        $row                                    = $request->getRow();
        $cdprato                                = $row['CDPRATO'];
        $cdFilAuxPrat                           = $row['CDFILAUXPRAT'];
        $produto                                = $row['CDPRODUTO'];
        isset($row['TPOPERACAO']) ? $tpOperacao = $row['TPOPERACAO'] : $tpOperacao = 'M'; 
        isset($row['PERCENTUAL']) ? $percentual = $row['PERCENTUAL'] : $percentual = '';
        isset($row['QTATIVA']) ? $qtAtiva       = $row['QTATIVA'] : $qtAtiva = '';
        if(isset($row['CDSERVFILI'])) 
            $row['CDSERVFILI']                      == '' ? $row['CDSERVFILI'] = null : $row['CDSERVFILI'] = $row['CDSERVFILI'];
        if(isset($row['CDFILIAL'])) 
            $row['CDFILIAL']                        == '' ? $row['CDFILIAL'] = null : $row['CDFILIAL'] = $row['CDFILIAL'];
        $tipoReceita                            = $row['TIPORECEITA'];
        try {
            $this->pratoService->iniciaTransacao();
            if ($tipoReceita === 'S') {
                $filial = $row['CDFILIAL'];
                $servico = $row['CDSERVFILI'];
                isset($row['IDQTDEFIXSE']) ? $qtAtiva = $row['IDQTDEFIXSE'] : $qtAtiva = null;
                
                $receita = $this->pratoService->retornaReceitaServico($cdprato, $cdFilAuxPrat, $filial, $servico);
                if ($receita){
                    
                    foreach ($receita as $produtos) {
                        if ($produtos->getCdproduto() <> $produto) {
                            if ($produtos->getIdqtdefixse() <> 'S') {
                                $quantidadesAtualizadas
                                    = $this->calcAlteracaoProporcional(
                                    $qtAtiva,
                                    $produtos->getQtprodrecser(),
                                    $produtos->getQtbrutaseunr(),
                                    $produtos->getQtbrutase(),
                                    ($produtos->getVrpefatcocse() + 100),
                                    (100 - $produtos->getVrpefatcorse()),
                                    $produtos->getQtcalcnutrse(),
                                    $percentual, $tpOperacao
                                );
                                    
                                $produtos->setQtprodrecser($quantidadesAtualizadas['QTLIQUIDA']);
                                $produtos->setQtbrutaseunr($quantidadesAtualizadas['QTBRUTA']);
                                $produtos->setQtbrutase($quantidadesAtualizadas['QTDREQUERIDA']);
                                $produtos->setQtcalcnutrse($quantidadesAtualizadas['QTNUTRICIONAL']);

                                $this->pratoService->flush();
                            }
                        }
                    }
                }
            }
            else if ($tipoReceita === 'L') {
                $filial = $row['CDFILIAL'];
                isset($row['IDQTDEFIXLO']) ? $qtAtiva = $row['IDQTDEFIXLO'] : $qtAtiva = null;
                
                $receita = $this->pratoService->retornaReceitaLocal($cdprato, $cdFilAuxPrat, $filial);
                if ($receita){
                    
                    foreach ($receita as $produtos) {
                        if ($produtos->getCdproduto() <> $produto) {
                            if ($produtos->getIdqtdefixlo() <> 'S') {
                                $quantidadesAtualizadas
                                    = $this->calcAlteracaoProporcional(
                                    $qtAtiva,
                                    $produtos->getQtprodrecloc(),
                                    $produtos->getQtbrutalounr(),
                                    $produtos->getQtbrutalo(),
                                    ($produtos->getVrpefatcoclo() + 100),
                                    (100 - $produtos->getVrpefatcorlo()),
                                    $produtos->getQtcalcnutrlo(),
                                    $percentual, $tpOperacao
                                );

                                $produtos->setQtprodrecloc($quantidadesAtualizadas['QTLIQUIDA']);
                                $produtos->setQtbrutalounr($quantidadesAtualizadas['QTBRUTA']);
                                $produtos->setQtbrutalo($quantidadesAtualizadas['QTDREQUERIDA']);
                                $produtos->setQtcalcnutrlo($quantidadesAtualizadas['QTNUTRICIONAL']);

                                $this->pratoService->flush();
                            }
                        }
                    }
                }
            }
            else if($tipoReceita === 'P'){
                $receita = $this->pratoService->retornaReceitaPadrao($cdprato, $cdFilAuxPrat);
                isset($row['IDQTDEFIXPR']) ? $qtAtiva = $row['IDQTDEFIXPR'] : $qtAtiva = null;

                if ($receita){
                    
                    foreach ($receita as $produtos) {
                        if ($produtos->getCdproduto() <> $produto) {
                            if ($produtos->getIdqtdefixpr() <> 'S') {
                                $quantidadesAtualizadas
                                    = $this->calcAlteracaoProporcional(
                                    $qtAtiva,
                                    $produtos->getQtprodprat(),
                                    $produtos->getQtbrutaprunr(),
                                    $produtos->getQtbrutapr(),
                                    ($produtos->getVrpefatcocpr() + 100),
                                    (100 - $produtos->getVrpefatcorpr()),
                                    $produtos->getQtcalcnutrpr(),
                                    $percentual, $tpOperacao
                                );
                                $produtos->setQtprodprat($quantidadesAtualizadas['QTLIQUIDA']);
                                $produtos->setQtbrutaprunr($quantidadesAtualizadas['QTBRUTA']);
                                $produtos->setQtbrutapr($quantidadesAtualizadas['QTDREQUERIDA']);
                                $produtos->setQtcalcnutrpr($quantidadesAtualizadas['QTNUTRICIONAL']);

                                $this->pratoService->flush();
                            }
                        }
                    }
                }
            } 
            else if($tipoReceita === 'Q'){
                $nrpadraoqualidade = $row['NRPADRAOQUALIDADE'];
                $conditions = array(
                    array(
                        "columnName" => "CDPRATO",
                        "operator" => "=",
                        "value" => $cdprato
                    ),
                    array(
                        "columnName" => "CDFILAUXPRAT",
                        "operator" => "=",
                        "value" => $cdFilAuxPrat
                    ),
                    array(
                        "columnName" => "NRORG",
                        "operator" => "=",
                        "value" => $this->environment->getNrOrgTrab()
                    ),
                    array(
                        "columnName" => "NRPADRAOQUALIDADE",
                        "operator" => "=",
                        "value" =>  $nrpadraoqualidade
                    )
                );
                
                $filterCriteria = new \Zeedhi\Framework\DataSource\FilterCriteria('recepadquali', $conditions);
                $dataSet = $this->dataSourceManager->findBy($filterCriteria);
                $receita = $dataSet->getRows();
                
                isset($row['IDQTDEFIXPR']) ? $qtAtiva = $row['IDQTDEFIXPR'] : $qtAtiva = null;
                if ($receita){
                    
                    foreach ($receita as $produtos) {

                        if ($produtos['CDPRODUTO'] <> $produto) {
                            if ($produtos['IDQTDEFIXPADQ'] <> 'S') {
                                                       
                                $quantidadesAtualizadas
                                    = $this->calcAlteracaoProporcional(
                                    $qtAtiva,
                                    $produtos['QTPRODRECPADQ'],
                                    $produtos['QTBRUTAPADQUNR'],
                                    0,
                                    ($produtos['VRPEFATCOCPADQ'] + 100),
                                    (100 - $produtos['VRPEFATCORPADQ']),
                                    0,
                                    $percentual, $tpOperacao
                                );
                                
                                
                                $produtos['QTPRODRECPADQ'] = $quantidadesAtualizadas['QTLIQUIDA'];
                                $produtos['QTBRUTAPADQUNR'] = $quantidadesAtualizadas['QTBRUTA'];
                                                      
                                $this->crudService->save('recepadquali',array($produtos));
                            }
                        }
                    }
                }
            }
            else if($tipoReceita === 'R'){
                $nrpadraoqualidade = $row['NRPADRAOQUALIDADE'];
                $cdregiao = $row['CDREGIAO'];
                $conditions = array(
                    array(
                        "columnName" => "CDPRATO",
                        "operator" => "=",
                        "value" => $cdprato
                    ),
                    array(
                        "columnName" => "CDFILAUXPRAT",
                        "operator" => "=",
                        "value" => $cdFilAuxPrat
                    ),
                    array(
                        "columnName" => "NRORG",
                        "operator" => "=",
                        "value" => $this->environment->getNrOrgTrab()
                    ),
                    array(
                        "columnName" => "NRPADRAOQUALIDADE",
                        "operator" => "=",
                        "value" =>  $nrpadraoqualidade
                    ),
                    array(
                        "columnName" => "CDREGIAO",
                        "operator" => "=",
                        "value" =>  $cdregiao
                    )
                );
                
                $filterCriteria = new \Zeedhi\Framework\DataSource\FilterCriteria('receregiao', $conditions);
                $dataSet = $this->dataSourceManager->findBy($filterCriteria);
                $receita = $dataSet->getRows();
                isset($row['IDQTDEFIXPR']) ? $qtAtiva = $row['IDQTDEFIXPR'] : $qtAtiva = null;
                
                if ($receita){
                    
                    foreach ($receita as $produtos) {
                        
                        if ($produtos['CDPRODUTO'] <> $produto) {
                            if ($produtos['IDQTDEFIXREGIAO'] <> 'S') {
                                $quantidadesAtualizadas
                                    = $this->calcAlteracaoProporcional(
                                    $qtAtiva,
                                    $produtos['QTPRODRECREGIAO'],
                                    $produtos['QTBRUTAREGIAOUNR'],
                                    0,
                                    ($produtos['VRPEFATCOCREGIAO'] + 100),
                                    (100 - $produtos['VRPEFATCORREGIAO']),
                                    0,
                                    $percentual, $tpOperacao
                                );
                                
                                $produtos['QTPRODRECREGIAO'] = $quantidadesAtualizadas['QTLIQUIDA'];
                                $produtos['QTBRUTAREGIAOUNR'] = $quantidadesAtualizadas['QTBRUTA'];
                                
                                $this->crudService->save('receregiao',array($produtos));
                            }
                        }
                    }
                }
            }
            $this->pratoService->commit();
            $response->addDataset(new DataSet_Source('CALLBACK', array(true)));
        } catch (\Exception $e) {
            $this->pratoService->rollback();
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }

    }


    protected function calcAlteracaoProporcional($campoAtivo, $qtdeLiq, $qtdeBruta, $qtdeReq, $aprovCoc, $aprovLimp, $qtdeNutr, $proporcaoQuantidade, $tpOperacao = 'M'){  
        if ($tpOperacao === 'M') { // em caso de multiplicacao
            $quantidadeReq = $qtdeReq * $proporcaoQuantidade;
            $quantidadeNut = $qtdeNutr * $proporcaoQuantidade;
            
            $quantidadeLiq = $qtdeLiq * $proporcaoQuantidade;
            $quantidadeBru = $qtdeBruta * $proporcaoQuantidade;
        } else { // em caso de divisão
            $quantidadeReq = $qtdeReq / $proporcaoQuantidade;
            $quantidadeNut = $qtdeNutr / $proporcaoQuantidade;

            $quantidadeLiq = $qtdeLiq / $proporcaoQuantidade;
            $quantidadeBru = $qtdeBruta / $proporcaoQuantidade;
        }

        $return = array(
            'QTDREQUERIDA'  => $quantidadeReq,
            'QTNUTRICIONAL' => $quantidadeNut,
            'QTLIQUIDA'     => $quantidadeLiq,
            'QTBRUTA'       => $quantidadeBru
        );
        return $return;
        
    }


    protected function calculaQuantidade($campoAtivo, $porcentagem, $quantidade){
        if($campoAtivo == 'L'){
            $novoValor = ($quantidade * $porcentagem) / 100;
        } else{
            $novoValor = ($quantidade / $porcentagem) * 100;
        }

        return $novoValor;
    }


    protected function getQuantidadeAtiva($campoAtivo, $qtdeLiq, $qtdeBruta){
        if($campoAtivo == 'B'){
            return $qtdeLiq;
        } else{
            return $qtdeBruta;
        }
    }


    protected function getCampoBloq($campoAtivo) {
        if ($campoAtivo == 'L') {
            return 'QTLIQUIDA';
        } else {
            return 'QTBRUTA';
        }
    }


    public function insereDadosPrato(
        $filAuxPrat, $prato, $filial, $servico, $corPri, $corSeq, $corAux,
        $quantidade, $container, $hora, $nomeFant, $qtCarb, $qtLip, $qtProt,
        $qtCal, $qtAlc, $qtMin, $qtAmost, $prodProd, $qtLim, $idProdChill,
        $idTpProdChill, $idFinal, $nrorg, $cdregiao = null, $nrpadraoqualidade = null, $code = null
    ){
        if ($servico) {
            $dadosPratInserir = array(
                'NRORG' => $nrorg,
                '__is_new' => true,
                'CDFILAUXPRAT' => $filAuxPrat,
                'CDPRATO' => $prato,
                'CDFILIAL' => $filial,
                'CDSERVFILI' => $servico,
                'NRCORPRISERV' => $corPri,
                'NRCORSEQSERV' => $corSeq,
                'NRCORAUXSERV' => $corAux,
                'QTPADRAOSERV' => $quantidade,
                'QTCONTAISERV' => $container,
                'HRVALCONSERV' => $hora,
                'NMFANTASIASE' => $nomeFant,
                'QTCARBOPRSER' => $qtCarb,
                'QTLIPIDPRSER' => $qtLip,
                'QTPROTEPRSER' => $qtProt,
                'QTCALORPRSER' => $qtCal,
                'QTALCOOPRSER' => $qtAlc,
                'VRPEMINSERV' => $qtMin,
                'QTAMOSTRSERV' => $qtAmost,
                'CDPRODPRODUCAO' => $prodProd,
                'QTLIMSEGSERV' => $qtLim,
                'IDPRODCHILL' => $idProdChill,
                'IDTPPRODCHILL' => $idTpProdChill,
                'IDFINALSERV' => $idFinal
            );

            $tabela = "pratserv";
        } else if($nrpadraoqualidade){
            if($cdregiao){
                $dadosPratInserir = array(
                    'NRORG'            => $nrorg,
                    '__is_new'         => true,
                    'CDFILAUXPRAT'     => $filAuxPrat,
                    'CDPRATO'          => $prato,
                    'NRPRATREGIAO'     => $code,
                    'NRPADRAOQUALIDADE'=> $nrpadraoqualidade,
                    'CDREGIAO'         => $cdregiao,
                    'NRCORPRIPRAT'     => $corPri,
                    'QTPADRAOPRAT'     => $quantidade
                );
    
                $tabela = "pratregiao";
            } else{
                $dadosPratInserir = array(
                    'NRORG'             => $nrorg,
                    '__is_new'          => true,
                    'CDFILAUXPRAT'      => $filAuxPrat,
                    'CDPRATO'           => $prato,
                    'NRPRATPADQUALI'    => $code,
                    'NRPADRAOQUALIDADE' => $nrpadraoqualidade,
                    'NRCORPRIPADQ'      => $corPri,
                    'QTPADRAOPADQ'      => $quantidade
                );
    
                $tabela = "pratpadquali";
            }
        } else if ($filial){
            $dadosPratInserir = array(
                'NRORG' => $nrorg,
                '__is_new' => true,
                'CDFILAUXPRAT' => $filAuxPrat,
                'CDPRATO' => $prato,
                'CDFILIAL' => $filial,
                'NRCORPRIFILI' => $corPri,
                'NRCORSEQFILI' => $corSeq,
                'NRCORAUXFILI' => $corAux,
                'QTPADRAOFILI' => $quantidade,
                'QTCONTAIFILI' => $container,
                'HRVALCONFILI' => $hora,
                'NMFANTASIAPF' => $nomeFant,
                'QTCARBOPRFIL' => $qtCarb,
                'QTLIPIDPRFIL' => $qtLip,
                'QTPROTEPRFIL' => $qtProt,
                'QTCALORPRFIL' => $qtCal,
                'QTALCOOPRFIL' => $qtAlc,
                'VRPEMINFILI' => $qtMin,
                'QTAMOSTRFILI' => $qtAmost,
                'CDPRODPRODUCAO' => $prodProd,
                'QTLIMSEGFILI' => $qtLim,
                'IDPRODCHILL' => $idProdChill,
                'IDTPPRODCHILL' => $idTpProdChill,
                'IDFINALFILI' => $idFinal
            );

            $tabela = "pratfili";

        } else {
            $dadosPratInserir = array(
                'NRORG' => $nrorg,
                '__is_new' => true,
                'CDFILAUXPRAT' => $filAuxPrat,
                'CDPRATO' => $prato,
                'NRCORPRIPRAT' => $corPri,
                'NRCORSEQPRAT' => $corSeq,
                'NRCORAUXPRAT' => $corAux,
                'QTPADRAOPRAT' => $quantidade,
                'QTCONTAIPRAT' => $container,
                'HRVALCONPRAT' => $hora,
                'NMFANTASIAPR' => $nomeFant,
                'QTCARBOPRPAD' => $qtCarb,
                'QTLIPIDPRPAD' => $qtLip,
                'QTPROTEPRPAD' => $qtProt,
                'QTCALORPRPAD' => $qtCal,
                'QTALCOOPRPAD' => $qtAlc,
                'VRPEMINPADR' => $qtMin,
                'QTAMOSTRPADR' => $qtAmost,
                'CDPRODPRODUCAO' => $prodProd,
                'QTLIMSEGPADR' => $qtLim,
                'IDPRODCHILL' => $idProdChill,
                'IDTPPRODCHILL' => $idTpProdChill,
                'IDFINALPADR' => $idFinal
            );

            $tabela = "pratpadr";
        }

        $requestInsertDadosPrato = new DTO\Request\Row(
            $dadosPratInserir, '', '', ''
        );
        $dadosPratInserir = $this->crudService->getRowsToSave(
            $requestInsertDadosPrato, $tabela
        );

        $this->crudService->save($tabela, $dadosPratInserir);
    }


    public function insereReceita($row, $receitaOrigem, $receitaDestino){
        
        $dadosReceitaInserir = array(
            '__is_new'                                         => true,
            'NRORG'                                            => $row['NRORG'],
            'CDPRATO'                                          => $row['CDPRATO'],
            'CDFILIAL'                                         => $row['CDFILIAL'],
            'CDPRODUTO'                                        => $row['CDPRODUTO'],
            'SGUNIDADE'                                        => $row['SGUNIDADEPROD'],
            'CDSERVFILI'                                       => $row['CDSERVFILI'],
            'CDFILAUXPRAT'                                     => $row['CDFILAUXPRAT'],
            $this->keysMapping['QTBRUTA'][$receitaDestino]     => isset($row['QTREQUERIDA'])  ? $row['QTREQUERIDA']   : null,
            $this->keysMapping['QTPROD'][$receitaDestino]      => isset($row['QUANTIDADE'])   ? $row['QUANTIDADE']    : null,
            $this->keysMapping['VRPEFATNUT'][$receitaDestino]  => isset($row['APROVNUTRI'])   ? $row['APROVNUTRI']    : "0",
            $this->keysMapping['VRPEFATCOR'][$receitaDestino]  => isset($row['PERDALIMP'])    ? $row['PERDALIMP']     : "0",
            $this->keysMapping['QTUNIDEST'][$receitaDestino]   => isset($row['QTUNIDEST'])    ? $row['QTUNIDEST']     : null,
            'CDAPRERECE'                                       => isset($row['APRESENT'])     ? $row['APRESENT']      : null,
            $this->keysMapping['VRPEFATCOC'][$receitaDestino]  => isset($row['GANHOCOC'])     ? $row['GANHOCOC']      : "0",
            $this->keysMapping['QTLIMSEG'][$receitaDestino]    => isset($row['DIASLIM'])      ? $row['DIASLIM']       : null,
            $this->keysMapping['QTCALCNUTR'][$receitaDestino]  => isset($row['QTNUTRI'])      ? $row['QTNUTRI']       : null,
            $this->keysMapping['QTBRUTAUNR'][$receitaDestino]  => isset($row['QTBRUTA'])      ? $row['QTBRUTA']       : null,
            $this->keysMapping['QTCOMPLPROD'][$receitaDestino] => isset($row['QTCOMPL'])      ? $row['QTCOMPL']       : null,
            $this->keysMapping['HRTEMPPREP'][$receitaDestino]  => isset($row['HRTEMP'])       ? $row['HRTEMP']        : null,
            $this->keysMapping['IDQTDEFIX'][$receitaDestino]   => isset($row['QTFIXO'])       ? $row['QTFIXO']        : "N",
            'IDUTCOMPENTDIR'.$receitaDestino                   => isset($row['ENTDIR'])       ? $row['ENTDIR']        : "N",
            $this->keysMapping['IDTPREQPROD'][$receitaDestino] => isset($row['TPREQ'])        ? $row['TPREQ']         : "P",
            $this->keysMapping['IDPRODBASE'][$receitaDestino]  => isset($row['BASE'])         ? $row['BASE']          : "N",
            $this->keysMapping['CODIGO'][$receitaDestino]      => isset($row['CODIGO'])       ? $row['CODIGO']        : null,
            'NRPADRAOQUALIDADE'                                => isset($row['NRPADRAOQUALIDADE']) ? $row['NRPADRAOQUALIDADE'] : null,
            'CDREGIAO'                                         => isset($row['CDREGIAO'])     ? $row['CDREGIAO']      : null,
            'VRPERCDEGELO'                                     => isset($row['VRPERCDEGELO']) ? $row['VRPERCDEGELO']  : "0"
        );
        switch ($receitaDestino) {
            case 'R':
                $tabela = 'receregiao';
            break;
            case 'Q':
                $tabela = 'recepadquali';
            break;
            case 'P':
                $tabela = 'recepadr';
            break;
            case 'S':
                $tabela = 'receserv';
            break;
            case 'L':
                $tabela = 'recefili';
            break;
        }


        $requestInsertReceitaPrato = new DTO\Request\Row(
            $dadosReceitaInserir, '', '', ''
        );
        $dadosReceitaInserir = $this->crudService->getRowsToSave(
            $requestInsertReceitaPrato, $tabela
        );
        
        $this->crudService->save($tabela, $dadosReceitaInserir);
    }


    public function retornaProdutoProducao(
        DTO\Request\Row $request,
        DTO\Response $response
    ){
        $row        = $request->getRow();
        $prato      = $row['CDPRATO'];
        $filauxprat = $row['CDFILAUXPRAT'];
        $produto    = $row['CDPRODUTO'];
        $tProduto   = $row['TCDPRODUTO'];
        $nrorg      = $this->environment->getNrOrgTrab();

        $produtos   = $this->pratoService->retornaProdutoProducao($prato, $filauxprat, $produto, $tProduto, $nrorg);
        $response->addDataset(new DataSet_Source('PRODUTOS', array($produtos)));
    }


    public function verificaProdutoProducao(
        DTO\Request\Row $request,
        DTO\Response $response
    ){
        $row = $request->getRow();
        $prato      = $row['CDPRATO'];
        $filauxprat = $row['CDFILAUXPRAT'];
        $produto    = $row['CDPRODUTO'];
        $receita    = $row['TPRECEITA'];
        $cdfilial   = $row['CDFILIAL'];
        $cdservfili = $row['CDSERVFILI'];
        $nrorg      = $this->environment->getNrOrgTrab();
        $resultado  = $this->pratoService->verificarProdutoProducao($prato, $filauxprat, $produto, $receita, $nrorg);
        if (count($resultado) > 0) {
            $mensagem = "O produto de produção já está sendo usado na(s) seguinte(s) receita(s): <br/>";
            foreach ($resultado as $receita) {
                if ($receita['TIPO'] === 'Padrão')  $mensagem .= $receita['TIPO'];
                if ($receita['TIPO'] === 'Local')   $mensagem .= $receita['TIPO'].' (Filial: '.$receita['CDFILIAL'].' - '.$receita['NMFILIAL'];
                if ($receita['TIPO'] === 'Serviço') $mensagem .= $receita['TIPO'].' (Filial: '.$receita['CDFILIAL'].' - '.$receita['NMFILIAL'].' | Serviço: '.$receita['CDSERVFILI'].' - '.$receita['NMSERVICO'];

                $mensagem .= ' | Prato: '.$receita['CDPRATO'].' - '.$receita['NMPRATO'].')<br/>';
            }

            $mensagem .= "<br /> Realize a importaçao da receita para utilização deste produto acabado.";
            $retorno = $mensagem;
        } else {
            $retorno = true;
        }
        $response->addDataset(new DataSet_Source('VERIFICACAO', array($retorno)));
    }

    //Utilizacao de produtos na receita
    public function alteracaoAutomatica(
        DTO\Request\Row $request,
        DTO\Response $response
    )
    {
        try {
            $this->pratoService->iniciaTransacao();
            $rows = $request->getRow();

            $listaProdutos = $rows['PRODUTOS'];
            $valores = $rows['VALORES'];
            $qtAtiva = $rows['QTATIVA'];

            $nrorg = $this->environment->getNrOrgTrab();
            $data = $this->environment->getCurrentTime();
            $operador = $this->environment->getCdOperador();

            foreach ($listaProdutos as $produtos) {
                $tabela = $produtos['NMTABLE'];
                $produtoIncl = $valores['CDPRODUTO'];

                if ($produtoIncl) {
                    $produtoMarcado = $produtoIncl;
                } else {
                    $produtoMarcado = $produtos['CDPRODUTO'];
                }

                $prato = $produtos['CDPRATO'];
                $filAuxPrato = $produtos['CDFILAUXPRAT'];

                $qtLiq    = $qtBruta    = $qtReq = $ganho = $perda = '0';
                $aprovCoc = $aprovPerda = $aprovNutri = '100';
                $unidade  = $qtNutri    = $tempo = '';

                switch ($tabela) {
                case 'RECEPADR':
                    
                    $produto = $this->retornaInformacoes(
                        strtolower($tabela), $prato, $filAuxPrato, null, null, $produtoMarcado
                    );
                    if ($produto && count($produto) > 0 ) {
                        $produto    = $produto[0];
                        $unidade    = $produto['SGUNIDADE'];
                        $qtLiq      = $produto['QTPRODPRAT'];
                        $qtBruta    = $produto['QTBRUTAPRUNR'];
                        $qtReq      = $produto['QTBRUTAPR'];
                        $ganho      = $produto['VRPEFATCOCPR'];
                        $aprovCoc   = $produto['VRPEFATCOCPR'] + 100;
                        $perda      = $produto['VRPEFATCORPR'];
                        $aprovPerda = 100 - $produto['VRPEFATCORPR'];
                        $qtNutri    = $produto['QTCALCNUTRPR'];
                        $aprovNutri = $produto['VRPEFATNUTPR'];
                        $tempo      = $produto['HRTEMPPREPPR'];
                        $vrdegelo   = $produto['VRPERCDEGELO'];
                    }
                    
                    break;
                case 'RECEFILI':
                    
                    $produto = $this->retornaInformacoes(
                        strtolower($tabela), $prato, $filAuxPrato, $produtos['FILIAL'], null,
                        $produtoMarcado
                    );
                    if ($produto && count($produto) > 0 )  {
                        $produto    = $produto[0];
                        $unidade    = $produto['SGUNIDADE'];
                        $qtLiq      = $produto['QTPRODRECLOC'];
                        $qtBruta    = $produto['QTBRUTALOUNR'];
                        $qtReq      = $produto['QTBRUTALO'];
                        $ganho      = $produto['VRPEFATCOCLO'];
                        $aprovCoc   = $produto['VRPEFATCOCLO'] + 100;
                        $perda      = $produto['VRPEFATCORLO'];
                        $aprovPerda = 100 - $produto['VRPEFATCORLO'];
                        $qtNutri    = $produto['QTCALCNUTRLO'];
                        $aprovNutri = $produto['VRPEFATNUTLO'];
                        $tempo      = $produto['HRTEMPPREPLO'];
                        $vrdegelo   = $produto['VRPERCDEGELO'];
                    }

                    break;
                case 'RECESERV':
                    
                    $produto = $this->retornaInformacoes(
                        strtolower($tabela), $prato, $filAuxPrato, $produtos['FILIAL'],
                        $produtos['CDSERVFILI'], $produtoMarcado
                    );
                    if ($produto && count($produto) > 0 ) {
                        $produto    = $produto[0];
                        $unidade    = $produto['SGUNIDADE'];
                        $qtLiq      = $produto['QTPRODRECSER'];
                        $qtBruta    = $produto['QTBRUTASEUNR'];
                        $qtReq      = $produto['QTBRUTASE'];
                        $ganho      = $produto['VRPEFATCOCSE'];
                        $aprovCoc   = $produto['VRPEFATCOCSE'] + 100;
                        $perda      = $produto['VRPEFATCORSE'];
                        $aprovPerda = 100 - $produto['VRPEFATCORSE'];
                        $qtNutri    = $produto['QTCALCNUTRSE'];
                        $aprovNutri = $produto['VRPEFATNUTSE'];
                        $tempo      = $produto['HRTEMPPREPSE'];
                        $vrdegelo   = $produto['VRPERCDEGELO'];
                    }

                    break;
                }
                
                $unAlt         = $valores['SGUNIDADEPROD'];
                $ganhoAlt      = $valores['VRPEFATCOCPR'];
                $perdaAlt      = $valores['VRPEFATCORPR'];
                $tempoAlt      = $valores['HRTEMPPREPPR'];
                $fatConv       = $valores['VRFATOCONVES'];
                $qtLiqAlt      = str_replace(',', '.', $valores['QTPRODPRAT']);
                $qtBrutaAlt    = str_replace(',', '.', $valores['QTBRUTAPRUNR']);
                $aprovGanhoAlt = str_replace(',', '.', $valores['VRPEFATCOCPRAP']);
                $aprovPerdaAlt = str_replace(',', '.', $valores['VRPEFATCORPRAP']);
                $aprovNutrAlt  = str_replace(',', '.', $valores['VRPEFATNUTPR']);
                $vrdegeloAlt   = str_replace(',', '.', $valores['VRPERCDEGELO']);

                if (empty($unAlt)) {
                    $unAlt = $unidade;
                }
                if (empty($qtLiqAlt)) {
                    $qtLiqAlt = $qtLiq;
                }
                if (empty($qtBrutaAlt)) {
                    $qtBrutaAlt = $qtBruta;
                }
                if (empty($ganhoAlt)) {
                    $ganhoAlt = $ganho;
                }
                if (empty($aprovGanhoAlt)) {
                    $aprovGanhoAlt = $aprovCoc;
                }
                if (empty($perdaAlt)) {
                    $perdaAlt = $perda;
                }
                if (empty($aprovPerdaAlt)) {
                    $aprovPerdaAlt = $aprovPerda;
                }
                if (empty($tempoAlt)) {
                    $tempoAlt = $tempo;
                }
                if (empty($aprovNutrAlt)) {
                    $aprovNutrAlt = $aprovNutri;
                }
                if (empty($vrdegeloAlt)) {
                    $vrdegeloAlt = $vrdegelo;
                }
                
                $qtdeCalc       = $this->getQuantidadeAtiva($qtAtiva, $qtLiqAlt, $qtBrutaAlt);
                $qtdGanho       = $this->calculaQuantidade($qtAtiva, $aprovGanhoAlt, $qtdeCalc);
                $qtdDegelo      = $this->calculaQuantidade($qtAtiva, 100-$vrdegeloAlt, $qtdGanho);
                $quantidadeFinal= $this->calculaQuantidade($qtAtiva, $aprovPerdaAlt, $qtdDegelo);
                
                if ($qtAtiva === 'L') {
                    $qtBruta = $qtBrutaAlt;
                    $qtLiq = $quantidadeFinal;
                } else {
                    $qtLiq = $qtLiqAlt;
                    $qtBruta = $quantidadeFinal;
                }
                
                
                if (empty($fatConv)) {
                    $fatConv = $this->retornaFatorConversao($produtoMarcado, $unAlt);   
                }

                $qtUniDest = $qtReq = $qtBruta * $fatConv;
                $qtCalcNutr = $qtBrutaAlt - ($qtReq * ($vrdegeloAlt / 100));
                $qtCalcNutr = $qtCalcNutr - ($qtCalcNutr * ($perdaAlt / 100));
                
                $qtLiq = str_replace(',', '.', $qtLiq);
                $qtBruta = str_replace(',', '.', $qtBruta);
                $ganhoAlt = str_replace(',', '.', $ganhoAlt);
                $perdaAlt = str_replace(',', '.', $perdaAlt);
                $qtUniDest = str_replace(',', '.', $qtUniDest);
                $qtReq = str_replace(',', '.', $qtReq);
                $qtCalcNutr = str_replace(',', '.', $qtCalcNutr);
                $aprovNutrAlt = str_replace(',', '.', $aprovNutrAlt);
                
               
                if (!$produto) { // insere
                    $produto = array();

                    $produto['CDPRODUTO'] = $produtoMarcado;
                    $produto['CDPRATO'] = $prato;
                    $produto['CDFILAUXPRAT'] = $filAuxPrato;
                }

                // atualiza Dados
                switch ($tabela) {
                case 'RECEPADR':
                    
                    $produto['SGUNIDADE']    = $unAlt;
                    $produto['QTPRODPRAT']   = floatval($qtLiq);
                    $produto['QTBRUTAPRUNR'] = floatval($qtBruta);
                    $produto['QTBRUTAPR']    = floatval($qtReq);
                    $produto['VRPEFATCOCPR'] = floatval($ganhoAlt);
                    $produto['VRPEFATCORPR'] = floatval($perdaAlt);
                    $produto['QTCALCNUTRPR'] = floatval($qtCalcNutr);
                    $produto['VRPEFATNUTPR'] = floatval($aprovNutrAlt);
                    $produto['QTUNIDESTPR']  = floatval($qtUniDest);
                    $produto['HRTEMPPREPPR'] = $tempoAlt;
                    $produto['VRPERCDEGELO'] = $vrdegeloAlt;
                    break;
                case 'RECEFILI':
                    $produto['SGUNIDADE']    = $unAlt;
                    $produto['QTPRODRECLOC'] = floatval($qtLiq);
                    $produto['QTBRUTALOUNR'] = floatval($qtBruta);
                    $produto['QTBRUTALO']    = floatval($qtReq);
                    $produto['VRPEFATCOCLO'] = floatval($ganhoAlt);
                    $produto['VRPEFATCORLO'] = floatval($perdaAlt);
                    $produto['QTCALCNUTRLO'] = floatval($qtCalcNutr);
                    $produto['VRPEFATNUTLO'] = floatval($aprovNutrAlt);
                    $produto['QTUNIDESTLO']  = floatval($qtUniDest);
                    $produto['HRTEMPPREPLO'] = $tempoAlt;
                    $produto['CDFILIAL']     = $produtos['FILIAL'];
                    $produto['VRPERCDEGELO'] = $vrdegeloAlt;
                    break;
                case 'RECESERV': 
                    $produto['SGUNIDADE']    = $unAlt;
                    $produto['QTPRODRECSER'] = floatval($qtLiq);
                    $produto['QTBRUTASEUNR'] = floatval($qtBruta);
                    $produto['QTBRUTASE']    = floatval($qtReq);
                    $produto['VRPEFATCOCSE'] = floatval($ganhoAlt);
                    $produto['VRPEFATCORSE'] = floatval($perdaAlt);
                    $produto['QTCALCNUTRSE'] = floatval($qtCalcNutr);
                    $produto['VRPEFATNUTSE'] = floatval($aprovNutrAlt);
                    $produto['QTUNIDESTSE']  = floatval($qtUniDest);
                    $produto['HRTEMPPREPSE'] = $tempoAlt;
                    $produto['CDFILIAL']     = $produtos['FILIAL'];
                    $produto['CDSERVFILI']   = $produtos['CDSERVFILI'];
                    $produto['VRPERCDEGELO'] = $vrdegeloAlt;
                    break;
                }
                $requestInsertPrat
                    = new \Zeedhi\Framework\DTO\Request\Row(
                    $produto, '', '', ''
                );
                $requestProduto = $this->crudService->getRowsToSave(
                    $requestInsertPrat, strtolower($tabela)
                );
                $this->crudService->save(strtolower($tabela), $requestProduto);
            }
            $this->pratoService->commit();
            $response->addDataset(new DataSet_Source('RETORNO', array(true)));
            return;
        } catch (\Exception $e) {
            $this->pratoService->rollback();
            $response->setError(
                new DTO\Response\Error($e->getMessage(), 1)
            );
        }
    }
    // FIM RECEITA


    // INICIO GERAL
    public function geraListaServico(
        DTO\Request\Row $request,
        DTO\Response $response
    ){
        try {
            $this->pratoService->iniciaTransacao();
            $row = $request->getRow();
            $agrupFili = $row['CDAGRUPFILI'];
            $filial = $row['CDFILIAL'];
            $servico = $row['CDSERVFILI'];
            
            // $codLista = $this->newCode->getCodeByTableName('LISTA', $this->environment->getNrOrgTrab());
            $data = $this->environment->getCurrentTime()->format('d/m/Y');
            
            if (is_array($servico)) {
                $codLista = $this->util->insertLista($servico);
            } else if ($filial) {
                if (is_array($filial)) { // BUSCA OS SERVIÇOS DE ACORDO COM A FILIAL SELECIONADA
                    foreach($filial as $fil) {
                        $condition = array(
                            array(
                                "columnName" => "CDFILIAL",
                                "operator" => "=",
                                "value" => $fil
                            ),
                            array(
                                "columnName" => "NRORG",
                                "operator" => "=",
                                "value" => $this->environment->getNrOrgTrab()
                            )
                        );
                        $filterCriteria = new \Zeedhi\Framework\DataSource\FilterCriteria('servico', $condition);
                        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
                        $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
                        $rows = $dataSet->getRows();
                        
                        $arrayLista = array();
                        foreach ($rows as $servico) {
                            array_push($arrayLista, $servico['CDFILIAL'].$servico['CDSERVFILI']);
                        }
                        $codLista = $this->util->insertLista($arrayLista);
                    }
                } else if ($filial != 'T') {
                    $condition = array(
                        array(
                            "columnName" => "CDFILIAL",
                            "operator" => "IN",
                            "value" => explode('_',$filial)
                        ),
                        array(
                            "columnName" => "NRORG",
                            "operator" => "=",
                            "value" => $this->environment->getNrOrgTrab()
                        )
                    );

                    $filterCriteria = new \Zeedhi\Framework\DataSource\FilterCriteria('servico', $condition);
                    $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
                    $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
                    $rows = $dataSet->getRows();
                    
                    $arrayLista = array();
                    foreach ($rows as $servico) {
                        array_push($arrayLista, $servico['CDFILIAL'].$servico['CDSERVFILI']);
                    }
                    $codLista = $this->util->insertLista($arrayLista);
                } else {
                    $conditions = array(
                        array(
                            "columnName" => "NRORG",
                            "operator" => "=",
                            "value" => $this->environment->getNrOrgTrab()
                        ),
                        array(
                            "columnName" => "CDOPERADOR",
                            "operator" => "=",
                            "value" => $this->environment->getCdOperador()
                        )
                    );
                    $dataSet = $this->crudService->getFilteredDataset("servico_filioper", $conditions);
                    $rows = $dataSet->getRows();
                    $arrayLista = array();
                    foreach ($rows as $servico) {
                        array_push($arrayLista, $servico["CDFILISERV"]);
                    }
                    $codLista = $this->util->insertLista($arrayLista);
                }
            } else if ($agrupFili) { // BUSCA AS FILIAIS DE ACORDO COM O AGRUPAMENTO FILTRADO E DEPOIS OS SERVICOS
                $condition = array(
                    array(
                        "columnName" => "NRORG",
                        "operator" => "=",
                        "value" => $this->environment->getNrOrgTrab()
                    ),
                    array(
                        "columnName" => "CDGRUPFILI",
                        "operator" => "=",
                        "value" => $agrupFili
                    ),
                    array(
                        "columnName" => "CDOPERADOR",
                        "operator" => "=",
                        "value" => $this->environment->getCdOperador()
                    )
                );

                $filterCriteria = new \Zeedhi\Framework\DataSource\FilterCriteria('filialgrupfili', $condition);
                $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
                $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
                $filiais = $dataSet->getRows();

                foreach ($filiais as $filial) {
                    $condition = array(
                        array(
                            "columnName" => "CDFILIAL",
                            "operator" => "=",
                            "value" => $filial['CDFILIAL']
                        ),
                        array(
                            "columnName" => "NRORG",
                            "operator" => "=",
                            "value" => $this->environment->getNrOrgTrab()
                        )
                    );

                    $filterCriteria = new \Zeedhi\Framework\DataSource\FilterCriteria('servico', $condition);
                    $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
                    $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
                    $rows = $dataSet->getRows();
                    
                    $arrayLista = array();
                    foreach ($rows as $servico) {
                        array_push($arrayLista, $servico['CDFILIAL'].$servico['CDSERVFILI']);
                    }
                    $codLista = $this->util->insertLista($arrayLista);
                }
            }

            $this->pratoService->commit();
            $response->addDataset(new DataSet_Source('LISTA', array($codLista)));
        } catch (\Exception $e) {
            $this->pratoService->rollback();
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }
    // FIM GERAL
    public function getMaxLevel ($params){
        try {
            $sql = "SELECT MAX(CDNVPRODUTO) AS MAXLEVEL
                        FROM PRODUTO P 
                        JOIN PRODFILI PF 
                          ON PF.CDPRODUTO = P.CDPRODUTO 
                          AND PF.NRORG = P.NRORG 
                        WHERE (PF.IDPRODATIV = :IDPRODATIV 
                               OR NVL(:IDPRODATIV, 'T') = 'T') 
                        AND P.NRORG = :NRORG";
            $parameter = array(
                'NRORG'      => $params['NRORG'],
                'IDPRODATIV' => $params['IDPRODATIV']
            );
            $conn = $this->pratoService->getConnection();
            return $conn->fetchAll($sql, $params);
        } catch (\Exception $e) {
            return null;
        }
    }
    
    //INÍCIO IMPORTAR FATOR DE CORRECAO PADRAO
    public function importarFatorCorrecaoPadrao(
        DTO\Request\Row $request,
        DTO\Response $response
    )
    {
        try {
        try {
            $this->pratoService->iniciaTransacao();
            $rows       = $request->getRow();
            $produtos   = $rows['PRODUTOS'];
            $filial     = $rows['CDFILIAL'];
            $prato      = $rows['CDPRATO'];
            $filAuxPrat = $rows['CDFILAUXPRAT'];
            $servico    = $rows['CDSERVFILI'];
            $tpReceita  = $rows['TPRECEITA'];
            $qtAtiva    = $rows['QTATIVA'];

            // Verifica se algum dos produtos está inativo na filial
            $conditions = array(
                array(
                    "columnName" => "CDPRATO",
                    "operator" => "=",
                    "value" => $prato
                ),
                array(
                    "columnName" => "CDFILAUXPRAT",
                    "operator" => "=",
                    "value" => $filAuxPrat
                ),
                array(
                    "columnName" => "CDFILIAL",
                    "operator" => "=",
                    "value" => $filial
                ),
                array(
                    "columnName" => "CDSERVFILI",
                    "operator" => "=",
                    "value" => $servico ? $servico : null
                )
            );
            $filterCriteria = new \Zeedhi\Framework\DataSource\FilterCriteria('pla01300_verifproduto'.strtolower($tpReceita), $conditions);
            $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, array('CDPRATO','CDNVPRODUTO','CDFILAUXPRAT','CDFILIAL','CDSERVFILI'));
            $verifprodutos = $this->dataSourceManager->findBy($newFilterCriteria)->getRows();
            foreach ($verifprodutos as $produto) {
                if($produto['IDPRODATIVPR'] == 'N')
                    throw new \Exception('Receita não pode conter produto inativo.');
            }
            foreach ($produtos as $produto) {
                $cdproduto = isset($produto['CDPRODUTO']) ? $produto['CDPRODUTO'] : $produto;

                // Busca fator de correção padrão do produto
                $conditions = array(
                    array(
                        "columnName" => "CDPRODUTO",
                        "operator" => "=",
                        "value" => $cdproduto
                    ),
                    array(
                        "columnName" => "NRORG",
                        "operator" => "=",
                        "value" => $this->environment->getNrOrgTrab()
                    )
                );
                $dataSet = $this->crudService->getFilteredDataSet('produto', $conditions);
                $dadosProduto = $dataSet->getRows()[0];
                $fatorCorrecao = $dadosProduto['VRPEFATOCORR'];
                
                // Calcula valores considerando o fator de correção
                switch ($tpReceita) {
                    case 'P':
                        $conditions = array(
                            array(
                                "columnName" => "CDPRATO",
                                "operator" => "=",
                                "value" => $prato
                            ),
                            array(
                                "columnName" => "CDFILAUXPRAT",
                                "operator" => "=",
                                "value" => $filAuxPrat
                            ),
                            array(
                                "columnName" => "CDPRODUTO",
                                "operator" => "=",
                                "value" => $cdproduto
                            )
                        );
                        $filterCriteria = new \Zeedhi\Framework\DataSource\FilterCriteria('recepadr', $conditions);
                        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
                        $result = $dataSet->getRows()[0];
                        
                        $qtLiq      = $result['QTPRODPRAT'];
                        $qtBruta    = $result['QTBRUTAPRUNR'];
                        $aprovCoc   = $result['VRPEFATCOCPR'] + 100;
                        $perda      = $result['VRPEFATCORPR'];
                        $aprovPerda = 100 - $result['VRPEFATCORPR'];
                        $aprovNutri = $result['VRPEFATNUTPR'];
                        $degelo     = $result['VRPERCDEGELO'];
                        $requerida  = $result['QTBRUTAPR'];
                        $quantNutr  = $result['QTCALCNUTRPR'];
                        $sgunidade = isset($result['SGUNIDADE']) ? $result['SGUNIDADE'] : $dadosProduto['SGUNIDADE'];
                        break;
                    case 'L':
                        $conditions = array(
                            array(
                                "columnName" => "CDPRATO",
                                "operator" => "=",
                                "value" => $prato
                            ),
                            array(
                                "columnName" => "CDFILAUXPRAT",
                                "operator" => "=",
                                "value" => $filAuxPrat
                            ),
                            array(
                                "columnName" => "CDFILIAL",
                                "operator" => "=",
                                "value" => $filial
                            ),
                            array(
                                "columnName" => "CDPRODUTO",
                                "operator" => "=",
                                "value" => $cdproduto
                            )
                        );
                        $filterCriteria = new \Zeedhi\Framework\DataSource\FilterCriteria('recefili', $conditions);
                        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
                        $result = $dataSet->getRows()[0];
                        
                        $qtLiq      = $result['QTPRODRECLOC'];
                        $qtBruta    = $result['QTBRUTALOUNR'];
                        $aprovCoc   = $result['VRPEFATCOCLO'] + 100;
                        $perda      = $result['VRPEFATCORLO'];
                        $aprovPerda = 100 - $result['VRPEFATCORLO'];
                        $aprovNutri = $result['VRPEFATNUTLO'];
                        $degelo     = $result['VRPERCDEGELO'];
                        $requerida  = $result['QTBRUTALO'];
                        $quantNutr  = $result['QTCALCNUTRLO'];
                        $sgunidade  = isset($result['SGUNIDADE']) ? $result['SGUNIDADE'] : $dadosProduto['SGUNIDADE'];
                        break;
                    case 'S':
                        $conditions = array(
                            array(
                                "columnName" => "CDPRATO",
                                "operator" => "=",
                                "value" => $prato
                            ),
                            array(
                                "columnName" => "CDFILAUXPRAT",
                                "operator" => "=",
                                "value" => $filAuxPrat
                            ),
                            array(
                                "columnName" => "CDFILIAL",
                                "operator" => "=",
                                "value" => $filial
                            ),
                            array(
                                "columnName" => "CDSERVFILI",
                                "operator" => "=",
                                "value" => $servico
                            ),
                            array(
                                "columnName" => "CDPRODUTO",
                                "operator" => "=",
                                "value" => $cdproduto
                            )
                        );
                        $filterCriteria = new \Zeedhi\Framework\DataSource\FilterCriteria('receserv', $conditions);
                        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
                        $result = $dataSet->getRows()[0];
                        
                        $qtLiq      = $result['QTPRODRECSER'];
                        $qtBruta    = $result['QTBRUTASEUNR'];
                        $aprovCoc   = $result['VRPEFATCOCSE'] + 100;
                        $perda      = $result['VRPEFATCORSE'];
                        $aprovPerda = 100 - $result['VRPEFATCORSE'];
                        $aprovNutri = $result['VRPEFATNUTSE'];
                        $degelo     = $result['VRPERCDEGELO'];
                        $requerida  = $result['QTBRUTASE'];
                        $quantNutr  = $result['QTCALCNUTRSE'];
                        $sgunidade = isset($result['SGUNIDADE']) ? $result['SGUNIDADE'] : $dadosProduto['SGUNIDADE'];
                        break;
                    case 'R':
                        $conditions = array(
                            array(
                                "columnName" => "CDPRATO",
                                "operator" => "=",
                                "value" => $prato
                            ),
                            array(
                                "columnName" => "CDFILAUXPRAT",
                                "operator" => "=",
                                "value" => $filAuxPrat
                            ),
                            array(
                                "columnName" => "CDPRODUTO",
                                "operator" => "=",
                                "value" => $cdproduto
                            )
                        );
                        $filterCriteria = new \Zeedhi\Framework\DataSource\FilterCriteria('receregiao', $conditions);
                        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
                        $result = $dataSet->getRows()[0];
                        
                        $qtLiq      = $result['QTPRODRECREGIAO'];
                        $qtBruta    = $result['QTBRUTAREGIAOUNR'];
                        $aprovCoc   = $result['VRPEFATCOCREGIAO'] + 100;
                        $perda      = $result['VRPEFATCORREGIAO'];
                        $aprovPerda = 100 - $result['VRPEFATCORREGIAO'];
                        $aprovNutri = 0;
                        $degelo     = 0;
                        $requerida  = $result['QTBRUTAREGIAOUNR'];
                        $quantNutr  = 0;
                        $sgunidade = isset($result['SGUNIDADE']) ? $result['SGUNIDADE'] : $dadosProduto['SGUNIDADE'];
                        break;
                    case 'Q':
                        $conditions = array(
                            array(
                                "columnName" => "CDPRATO",
                                "operator" => "=",
                                "value" => $prato
                            ),
                            array(
                                "columnName" => "CDFILAUXPRAT",
                                "operator" => "=",
                                "value" => $filAuxPrat
                            ),
                            array(
                                "columnName" => "CDPRODUTO",
                                "operator" => "=",
                                "value" => $cdproduto
                            )
                        );
                        $filterCriteria = new \Zeedhi\Framework\DataSource\FilterCriteria('recepadquali', $conditions);
                        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
                        $result = $dataSet->getRows()[0];
                        
                        $qtLiq      = $result['QTPRODRECPADQ'];
                        $qtBruta    = $result['QTBRUTAPADQUNR'];
                        $aprovCoc   = $result['VRPEFATCOCPADQ'] + 100;
                        $perda      = $result['VRPEFATCORPADQ'];
                        $aprovPerda = 100 - $result['VRPEFATCORPADQ'];
                        $aprovNutri = 0;
                        $degelo     = 0;
                        $requerida  = $result['QTBRUTAPADQUNR'];
                        $quantNutr  = 0;
                        $sgunidade = isset($result['SGUNIDADE']) ? $result['SGUNIDADE'] : $dadosProduto['SGUNIDADE'];
                        break;
                }
                

                $perda = $fatorCorrecao;
                $aprovPerda = 100 - $perda;
                $fatorConv = $this->retornaFatorConversao($cdproduto, $sgunidade);
                $quantidade = $this->getQuantidadeAtiva($qtAtiva, $qtLiq, $qtBruta);
                
                $quantidadeGanho = $this->calculaQuantidade($qtAtiva, $aprovCoc, $quantidade);
                $quantidadeDegelo= $this->calculaQuantidade($qtAtiva, 100-$degelo, $quantidadeGanho);
                $quantidadeFinal = $this->calculaQuantidade($qtAtiva, $aprovPerda, $quantidadeDegelo);
                
                if ($qtAtiva == 'L') {
                    $qtLiq = $quantidadeFinal;
                    $requerida = $qtBruta * $fatorConv;
                } else {
                    $qtBruta = $quantidadeFinal;
                    $requerida = $quantidadeFinal * $fatorConv;
                }
                $quantNutr = $this->calculaAprovNutricional($aprovNutri, $perda, $requerida, $degelo);
             
                // atualiza valores
                switch ($tpReceita) {
                    case 'P':
                        $result['VRPEFATCORPR'] = $fatorCorrecao;
                        $result['QTPRODPRAT'] = $qtLiq;
                        $result['QTCALCNUTRPR'] = $quantNutr;
                        $result['QTBRUTAPRUNR'] = $qtBruta;
                        $result['QTBRUTAPR']    = $requerida;
                        $this->crudService->save('recepadr',array($result));
    
                        break;
                    case 'L':
                        $result['VRPEFATCORLO'] = $fatorCorrecao;
                        $result['QTPRODRECLOC'] = $qtLiq;
                        $result['QTCALCNUTRLO'] = $quantNutr;
                        $result['QTBRUTALOUNR'] = $qtBruta;
                        $result['QTBRUTALO']    = $requerida;
                        $this->crudService->save('recefili',array($result));
    
                        break;
                    case 'S':
                        $result['VRPEFATCORSE'] = $fatorCorrecao;
                        $result['QTPRODRECSER'] = $qtLiq;
                        $result['QTCALCNUTRSE'] = $quantNutr;
                        $result['QTBRUTASEUNR'] = $qtBruta;
                        $result['QTBRUTASE']    = $requerida;
                        $this->crudService->save('receserv',array($result));
                        break;
                    case 'R':
                        $result['VRPEFATCORREGIAO'] = $fatorCorrecao;
                        $result['QTPRODRECREGIAO']  = $qtLiq;
                        $result['QTBRUTAREGIAOUNR'] = $qtBruta;
                        $this->crudService->save('receregiao',array($result));
                        break;
                    case 'Q':
                        $result['VRPEFATCORPADQ'] = $fatorCorrecao;
                        $result['QTPRODRECPADQ']  = $qtLiq;
                        $result['QTBRUTAPADQUNR'] = $qtBruta;
                        $this->crudService->save('recepadquali',array($result));
                        break;
                    }
            }
            $this->pratoService->flush();
            $this->pratoService->commit();
            $response->addDataset(new DataSet_Source('RETORNO', array(true)));
            return;
        } catch (\Exception $e) {
            $this->pratoService->rollback();
            $response->setError(
                new DTO\Response\Error($e->getMessage(), 1)
            );
        }
    } catch (\Exception $e) {
            $this->pratoService->rollback();
            $response->setError(
                new DTO\Response\Error($e->getMessage(), 1)
            );
        }
    }
    //FIM IMPORTAR FATOR DE CORRECAO PADRAO

    public function retornaArvoreProdutoEstoque(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = "produto_niveis_e_estoque";
        $this->find($request, $response);
    }
    
    
    public function importarReceita(DTO\Request\Row $request, DTO\Response $response){

        $row = $request->getRow();
        $tpImportacao = $row['TIPO_IMPORT'];

        //DADOS DE ORIGEM
        $pratoOrigem             = $row['PRATO_O'];
        $cdFilAuxPratoOrigem     = $row['CDFILAUXPRAT_O'];
        $receitaOrigem           = $row['RECEITA_O'];
        $cdregiaoOrigem          = array_key_exists('CDREGIAO_O', $row)           ? $row['CDREGIAO_O'] : null;
        $nrpadraoqualidadeOrigem = array_key_exists('NRPADRAOQUALIDADE_O', $row)  ? $row['NRPADRAOQUALIDADE_O'] : null;
        $filialO                 = $receitaOrigem == "L" || $receitaOrigem == "S" ? $row['FILIAL_O'] : null;
        $servicoO                = $receitaOrigem == "S"                          ? $row['SERVICO_O'] : null;
        

        //DADOS DE DESTINO
        $pratoDestino             = $row['PRATO_D'];
        $cdFilAuxPratoDestino     = $row['CDFILAUXPRAT_D'];
        $receitaDestino           = $row['RECEITA_D'];
        $cdregiaoDestino          = array_key_exists('CDREGIAO_D', $row)            ? $row['CDREGIAO_D'] : null;
        $nrpadraoqualidadeDestino = array_key_exists('NRPADRAOQUALIDADE_D', $row)   ? $row['NRPADRAOQUALIDADE_D'] : null;
        $filialD                  = $receitaDestino == "L" || $receitaDestino == "S" ? $row['FILIAL_D'] : null;
        $servicoD                 = $receitaDestino == "S"                           ? $row['SERVICO_D'] : null;
        
        switch ($receitaDestino) {
            case 'R':
                $dataSourceDestino = 'receregiao';
                $dadosPratoDestino = 'pratregiao';
            break;
            case 'Q':
                $dataSourceDestino = 'recepadquali';
                $dadosPratoDestino = 'pratpadquali';
            break;
            case 'P':
                $dataSourceDestino = 'recepadr';
                $dadosPratoDestino = 'pratpadr';
            break;
            case 'S':
                $dataSourceDestino = 'receserv';
                $dadosPratoDestino = 'pratserv';
            break;
            case 'L':
                $dataSourceDestino = 'recefili';
                $dadosPratoDestino = 'pratfili';
            break;
        }
        
        switch ($receitaOrigem) {
            case 'R':
                $dataSourceOrigem = 'receregiao';
                $dadosPratoOrigem = 'pratregiao';
            break;
            case 'Q':
                $dataSourceOrigem = 'recepadquali';
                $dadosPratoOrigem = 'pratpadquali';
            break;
            case 'P':
                $dataSourceOrigem = 'recepadr';
                $dadosPratoOrigem = 'pratpadr';
            break;
            case 'S':
                $dataSourceOrigem = 'receserv';
                $dadosPratoOrigem = 'pratserv';
            break;
            case 'L':
                $dataSourceOrigem = 'recefili';
                $dadosPratoOrigem = 'pratfili';
            break;
        }
        
        try{
            $this->pratoService->iniciaTransacao();
            // BUSCA RECEITA DE DESTINO
            if($this->pratoService->validaImportacaoReceita($row)){
                $conditionsDestino = array( //filtro Destino
                    array(
                        "columnName" => "CDPRATO",
                        "operator" => "=",
                        "value" => $pratoDestino
                    ),
                    array(
                        "columnName" => "CDFILAUXPRAT",
                        "operator" => "=",
                        "value" => $cdFilAuxPratoDestino
                    ),
                    array(
                        "columnName" => "NRORG",
                        "operator" => "=",
                        "value" => $this->environment->getNrOrgTrab()
                    )
                );
                if($receitaDestino == 'R' || $receitaDestino == 'Q'){
                    if($receitaDestino == 'R'){
                        array_push($conditionsDestino, 
                            array(
                                "columnName" => "CDREGIAO",
                                "operator" => "=",
                                "value" => $cdregiaoDestino
                            )
                        );
                    }
                    array_push($conditionsDestino, 
                        array(
                            "columnName" => "NRPADRAOQUALIDADE",
                            "operator" => "=",
                            "value" => $nrpadraoqualidadeDestino
                        )
                    );
                }else if($receitaDestino == 'S' || $receitaDestino == 'L'){
                    if($receitaDestino == 'S'){
                        array_push($conditionsDestino, 
                            array(
                                "columnName" => "CDSERVFILI",
                                "operator" => "=",
                                "value" => $servicoD
                            )
                        );
                    }
                    array_push($conditionsDestino, 
                        array(
                            "columnName" => "CDFILIAL",
                            "operator" => "=",
                            "value" => $filialD
                        )
                    );
                }
                
                $filterCriteria = new \Zeedhi\Framework\DataSource\FilterCriteria($dataSourceDestino, $conditionsDestino);
                $dataSetProdRecD = $this->dataSourceManager->findBy($filterCriteria);
                $prodRec_D = $dataSetProdRecD->getRows();
    
                // BUSCA DADOS DA RECEITA DE DESTINO E EXCLUI
                $filterCriteria = new \Zeedhi\Framework\DataSource\FilterCriteria($dadosPratoDestino, $conditionsDestino);
                $dataSet = $this->dataSourceManager->findBy($filterCriteria);
                
                $this->dataSourceManager->delete($dataSet);
                // BUSCA A RECEITA DE ORIGEM, QUE SERÁ COPIADA
                
                $conditionsOrigem = array( // Filtro Origem
                    array(
                        "columnName" => "CDPRATO",
                        "operator" => "=",
                        "value" => $pratoOrigem
                    ),
                    array(
                        "columnName" => "CDFILAUXPRAT",
                        "operator" => "=",
                        "value" => $cdFilAuxPratoOrigem
                    ),
                    array(
                        "columnName" => "NRORG",
                        "operator" => "=",
                        "value" => $this->environment->getNrOrgTrab()
                    )
                );
                
                if($receitaOrigem == 'R' || $receitaOrigem == 'Q'){
                    if($receitaOrigem == 'R'){
                        array_push($conditionsOrigem, 
                            array(
                                "columnName" => "CDREGIAO",
                                "operator" => "=",
                                "value" => $cdregiaoOrigem
                            )
                        );
                    }
                    array_push($conditionsOrigem, 
                        array(
                            "columnName" => "NRPADRAOQUALIDADE",
                            "operator" => "=",
                            "value" => $nrpadraoqualidadeOrigem
                        )
                    );
                }else if($receitaOrigem == 'S' || $receitaOrigem == 'L'){
                    if($receitaOrigem == 'S'){
                        array_push($conditionsOrigem, 
                            array(
                                "columnName" => "CDSERVFILI",
                                "operator" => "=",
                                "value" => $servicoO
                            )
                        );
                    }
                    array_push($conditionsOrigem, 
                        array(
                            "columnName" => "CDFILIAL",
                            "operator" => "=",
                            "value" => $filialO
                        )
                    );
                }
                
                $filterCriteria = new \Zeedhi\Framework\DataSource\FilterCriteria($dadosPratoOrigem, $conditionsOrigem);
                $dataSet = $this->dataSourceManager->findBy($filterCriteria);
                $pratpadrO = $dataSet->getRows();
                if ($tpImportacao === 'A') { // ACRESCENTA
                    if ($prodRec_D) {
                        foreach ($prodRec_D as $prodD) {
                            foreach ($row['NOVOSPRODUTOS'] as $indice => $prodO) {
                                if ($prodO['CDPRODUTO'] === $prodD['CDPRODUTO']) {
                                    unset($row['NOVOSPRODUTOS'][$indice]);
                                    continue 1;
                                }
                            }
                        }
                    }
                } else { // SOBREPOE
                    $this->dataSourceManager->delete($dataSetProdRecD);
                }
                
                foreach ($row['NOVOSPRODUTOS'] as $novosProdutos) {
                    if($receitaDestino == 'R' || $receitaDestino == 'Q'){
                        $code = $this->newCode->getCodeByTableName(strtoupper($dataSourceDestino), $this->environment->getNrOrgTrab());
                        $novosProdutos['CODIGO'] = $code;
                    }
                    $novosProdutos['CDPRATO'] = $pratoDestino;
                    $novosProdutos['CDFILAUXPRAT'] = $cdFilAuxPratoDestino;
                    $novosProdutos['CDREGIAO'] = $cdregiaoDestino;
                    $novosProdutos['NRPADRAOQUALIDADE'] = $nrpadraoqualidadeDestino;
                    $this->insereReceita($novosProdutos, $receitaOrigem, $receitaDestino);
                }
                if ($pratpadrO){
                    $pratpadrO     = $pratpadrO[0];
                    $corAux        = null;
                    $corSeq        = null;
                    $container     = null;
                    $hora          = null;
                    $nomeFant      = null;
                    $qtCarb        = null;
                    $qtLip         = null;
                    $qtProt        = null;
                    $qtCal         = null;
                    $qtAlc         = null;
                    $qtMin         = null;
                    $qtAmost       = null;
                    $prodProd      = isset($pratpadrO['CDPRODPRODUCAO']) ? $pratpadrO['CDPRODPRODUCAO'] : null;
                    $qtLim         = null;
                    $idProdChill   = isset($pratpadrO['IDPRODCHILL']) ? $pratpadrO['IDPRODCHILL'] : null;
                    $idTpProdChill = isset($pratpadrO['IDTPPRODCHILL']) ? $pratpadrO['IDTPPRODCHILL'] : null;
                    $idFinal       = null;
                    switch ($receitaOrigem) {
                        case 'R':
                            $cor        = $pratpadrO['NRCORPRIREGIAO'];
                            $quantidade = $pratpadrO['QTPADRAOREGIAO'];
                        break;
                        case 'Q':
                            $cor        = $pratpadrO['NRCORPRIPADQ'];
                            $quantidade = $pratpadrO['QTPADRAOPADQ'];
                        break;
                        case 'P':
                            $cor        = $pratpadrO['NRCORPRIPRAT'];
                            $quantidade = $pratpadrO['QTPADRAOPRAT'];
                            $corSeq     = $pratpadrO['NRCORSEQPRAT'];
                            $corAux     = $pratpadrO['NRCORAUXPRAT'];
                            $container  = $pratpadrO['QTCONTAIPRAT'];
                            $hora       = $pratpadrO['HRVALCONPRAT'];
                            $nomeFant   = $pratpadrO['NMFANTASIAPR'];
                            $qtCarb     = $pratpadrO['QTCARBOPRPAD'];
                            $qtLip      = $pratpadrO['QTLIPIDPRPAD'];
                            $qtProt     = $pratpadrO['QTPROTEPRPAD'];
                            $qtCal      = $pratpadrO['QTCALORPRPAD'];
                            $qtAlc      = $pratpadrO['QTALCOOPRPAD'];
                            $qtMin      = $pratpadrO['VRPEMINPADR'];
                            $qtAmost    = $pratpadrO['QTAMOSTRPADR'];
                            $qtLim      = $pratpadrO['QTLIMSEGPADR'];
                            $idFinal    = $pratpadrO['IDFINALPADR'];
                        break;
                        case 'S':
                            $cor         = $pratpadrO['NRCORPRISERV'];
                            $quantidade  = $pratpadrO['QTPADRAOSERV'];
                            $corSeq      = $pratpadrO['NRCORSEQSERV'];
                            $corAux      = $pratpadrO['NRCORAUXSERV'];
                            $container   = $pratpadrO['QTCONTAISERV'];
                            $hora        = $pratpadrO['HRVALCONSERV'];
                            $nomeFant    = $pratpadrO['NMFANTASIASE'];
                            $qtCarb      = $pratpadrO['QTCARBOPRSER'];
                            $qtLip       = $pratpadrO['QTLIPIDPRSER'];
                            $qtProt      = $pratpadrO['QTPROTEPRSER'];
                            $qtCal       = $pratpadrO['QTCALORPRSER'];
                            $qtAlc       = $pratpadrO['QTALCOOPRSER'];
                            $qtMin       = $pratpadrO['VRPEMINSERV'];
                            $qtAmost     = $pratpadrO['QTAMOSTRSERV'];
                            $qtLim       = $pratpadrO['QTLIMSEGSERV'];
                            $idFinal     = $pratpadrO['IDFINALSERV'];
                        break;
                        case 'L':
                            $cor        = $pratpadrO['NRCORPRIFILI'];
                            $quantidade = $pratpadrO['QTPADRAOFILI'];
                            $corSeq     = $pratpadrO['NRCORSEQFILI'];
                            $corAux     = $pratpadrO['NRCORAUXFILI'];
                            $container  = $pratpadrO['QTCONTAIFILI'];
                            $hora       = $pratpadrO['HRVALCONFILI'];
                            $nomeFant   = $pratpadrO['NMFANTASIAPF'];
                            $qtCarb     = $pratpadrO['QTCARBOPRFIL'];
                            $qtLip      = $pratpadrO['QTLIPIDPRFIL'];
                            $qtProt     = $pratpadrO['QTPROTEPRFIL'];
                            $qtCal      = $pratpadrO['QTCALORPRFIL'];
                            $qtAlc      = $pratpadrO['QTALCOOPRFIL'];
                            $qtMin      = $pratpadrO['VRPEMINFILI'];
                            $qtAmost    = $pratpadrO['QTAMOSTRFILI'];
                            $qtLim      = $pratpadrO['QTLIMSEGFILI'];
                            $idFinal    = $pratpadrO['IDFINALFILI'];
                        break;
                    }
                    
                    $code = $this->newCode->getCodeByTableName(strtoupper($dataSourceDestino), $this->environment->getNrOrgTrab());
                    $this->insereDadosPrato(
                        $cdFilAuxPratoDestino, $pratoDestino, $filialD, $servicoD, $cor, 
                        $corSeq, $corAux, $quantidade,  $container, $hora, $nomeFant, $qtCarb, $qtLip, $qtProt,
        $qtCal, $qtAlc, $qtMin, $qtAmost, $prodProd, $qtLim, $idProdChill,
        $idTpProdChill, $idFinal,
                        $pratpadrO['NRORG'], $cdregiaoDestino, $nrpadraoqualidadeDestino, $code
                    );
                }
                $this->pratoService->flush();
                $this->pratoService->commit();
            }else{
                if(isset($row['prodInativo']) && $row['prodInativo'] == true){
                    $response->setError(new DTO\Response\Error('Há produto inativo na receita.', 1));
                } else if(isset($row['estoque']) && $row['estoque'] == true){
                    $response->setError(new DTO\Response\Error('Há produto sem controle de estoque na receita.', 1));
                }
            }
        } catch (\Exception $e) {
            $this->pratoService->rollback();
            throw $e;
        }
    }
    
    public function salvaDadosReceitaPrato($request, $response){
        $row = $request->getRow();
        $tipoReceita = $row['TIPORECEITA'];
        $dadosPratoDestino = '';
        
        switch ($tipoReceita) {
            case 'R':
                $dadosPratoDestino = 'pratregiao';
            break;
            case 'Q':
                $dadosPratoDestino = 'pratpadquali';
            break;
            case 'P':
                $dadosPratoDestino = 'pratpadr';
            break;
            case 'S':
                $dadosPratoDestino = 'pratserv';
            break;
            case 'L':
                $dadosPratoDestino = 'pratfili';
            break;
        }
        
        $rows = $this->crudService->getRowsToSave($request, $dadosPratoDestino);
        $this->crudService->save($dadosPratoDestino, $rows);
    }
    
    public function retornaFotosPrato($request, $response){
        $conditions = $request->getFilterCriteria()->getConditions();
        $dataSourceName = '';
        foreach($conditions as $key => &$condition){
            if($condition['columnName'] == 'dataSource'){
                $dataSourceName = $condition['value'];
                unset($conditions[$key]);
            }
        }
        $filterCriteria = new \Zeedhi\Framework\DataSource\FilterCriteria($dataSourceName, $conditions);
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        $rows = $dataSet->getRows();
        foreach($rows as &$row){
            $row['FGFOTOPRAPAD'] = "data:image/jpg;base64,".$this->hex_to_base64($row['FGFOTOPRAPAD']);
        }
        $response->addDataset(new DataSet_Source('retornaFotosPrato', $rows));
    }
    
    function hex_to_base64($hex){
      $return = '';
      foreach(str_split($hex, 2) as $pair){
        $return .= chr(hexdec($pair));
      }
      return base64_encode($hex);
    }
    
    public function savePicture($request, $response){
        try{
            $row = $request->getRow();
            $insert = "INSERT INTO FOTOPRAPAD (CDFILAUXPRAT, CDPRATO, NRFOTOPRAPAD, IDAJUSFOTPAD, FGFOTOPRAPAD)
                       VALUES (:CDFILAUXPRAT, :CDPRATO, :NRFOTOPRAPAD, :IDAJUSFOTPAD, :FGFOTOPRAPAD)";

            $params = array(
                "CDFILAUXPRAT"     => $row['CDFILAUXPRAT'],
                "CDPRATO"          => $row['CDPRATO'],
                "NRFOTOPRAPAD"     => $row['NRFOTOPRAPAD'],
                "IDAJUSFOTPAD"     => $row['IDAJUSFOTPAD'],
                "FGFOTOPRAPAD"     => $row['FGFOTOPRAPAD']
            );
            $stmt = $this->connection->prepare($insert);
            $stmt->execute($params);
            
            // $rows = $this->crudService->getRowsToSave($request, $row['dataSourceName']);
            // $this->crudService->save($row['dataSourceName'], $rows);
        }catch(\Exception $e){
            throw $e;
        }
    }
    
    public function retornaParametrosAdicionarPrato($request, $response){
        $row = $request->getRow();
        $this->authController->validationDependingGroup($request, $response);
        
        $filterCriteria = new \Zeedhi\Framework\DataSource\FilterCriteria('paramgeral', array());
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        $response->addDataSet($dataSet);
    }
    
    
    //INICIO IMPORTAÇÃO PRATO XLS
    
    
    public function importaPratoXLS(DTO\Request\Row $request, DTO\Response $response) {
        try{
            $this->pratoService->iniciaTransacao();
            $rows = $request->getRow();
            $rowAnt = '';
            
            foreach ($rows as $row) {
                $row['CDOPERADOR'] = $this->environment->getCdOperador();
                $pratoExiste = $this->pratoService->retornaPrato($row['CDPRATO'], $row['CDFILAUXPRAT']);
                if (count($pratoExiste) > 0){
                    if($rowAnt != $row['CDPRATO']) $this->pratoService->excluiReceitaCompl('RECEPADR', 'PRATPADR', $row['CDPRATO'], $row['CDFILAUXPRAT']);
                    
                    $this->pratoService->atualizaPrato($row['CDFILAUXPRAT'], $row['CDPRATO'], $row['NMPRATO'], $row['SGUNIDADE']);
                } else {
                    $this->validaPais($row);
                    $this->pratoService->inserePratoPadrao($row);
                }
                
                $rowAnt = $row['CDPRATO'];
                if ($row['CDPRODUTO']) $this->pratoService->insereReceitaPadrao($row);
            }
            
            $this->pratoService->commit();
        }catch(\Exception $e){
            $this->pratoService->rollback();
            throw $e;
        }
    }
    
    protected function validaPais($info){
        $ultimoNivel = $info['CDNVPRATO'] - 1; 
        $pais = $this->pratoService->retornaPratosEpais(
            $info['CDFILAUXPRAT'], "CDPRATO = ".$info['CDPRATONIVANTE']);
               
        if (is_array($pais)) {
            if (count($pais) == $ultimoNivel) return true;
        }
         
        for ($totalPais = 1; $totalPais <= $ultimoNivel; $totalPais++) {
            if (count($pais) > 0 and $totalPais == $pais[$totalPais - 1]) continue;
            else {
                foreach ($info['MASCARA'] as $mascara) {
                    if ($mascara['nivel'] == $totalPais) {
                        $codigo = substr($info['CDPRATO'], 0, $mascara['caracteres']);
                        break;
                    }
                }  
                
                $pratoExiste = $this->pratoService->retornaPrato($codigo, $info['CDFILAUXPRAT']);        
                if (count($pratoExiste) == 0){
                    $dados = array("CDFILAUXPRAT"=> $info['CDFILAUXPRAT'],
                                    "CDPRATO"=> $codigo,
                                    "NMPRATO"=> $info['NMPRATO'],
                                    "CDNVPRATO"=> $totalPais,
                                    "SGUNIDADE"=> null,
                                    "NRORG"=> $info['NRORG']);
                                    
                    $this->pratoService->inserePratoPadrao($dados);
                }
            }
        }
    }
    
    //FIM IMPORTAÇÃO PRATO XLS
    
    private function retornaInformacoes($dataSourceName, $prato, $filAuxPrat, $filial = null, $servico = null, $produto = null) {
        $conditions = array(
            array(
                "columnName" => "CDPRATO",
                "operator" => "=",
                "value" => $prato
            ),
            array(
                "columnName" => "CDFILAUXPRAT",
                "operator" => "=",
                "value" => $filAuxPrat
            )
        );
        if($servico || $filial){
            if($servico){
                array_push($conditions, 
                    array(
                        "columnName" => "CDSERVFILI",
                        "operator" => "=",
                        "value" => $filial
                    )
                );
            }
            array_push($conditions, 
                array(
                    "columnName" => "CDFILIAL",
                    "operator" => "=",
                    "value" => $filial
                )
            );
        }
        if($produto) { 
            array_push($conditions, 
                array(
                    "columnName" => "CDPRODUTO",
                    "operator" => "=",
                    "value" => $produto
                )
            );
        }
        $filterCriteria = new \Zeedhi\Framework\DataSource\FilterCriteria($dataSourceName, $conditions);
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        return $dataSet->getRows();
    }
    
    private function retornaFatorConversao($produto, $sgunidade) {
        $fatorConversaoProduto = $this->pratoService->retornaFatorConversao($produto, $sgunidade, $this->environment->getNrOrgTrab());
        if($fatorConversaoProduto) {
            foreach ($fatorConversaoProduto as $fator){
                if($fator['SGUNIDADE'] == $sgunidade){
                    return $fator['VRFATOCONVES'];
                }
            }
            return 1;
        } else {
            return 1;
        }
    }
    
    private function calculaAprovNutricional($aprovNutri, $perdaLimp, $quantidadeBru, $degelo) {
        $baseNutri = $quantidadeBru - $this->porcentagem($quantidadeBru, $degelo);
        $baseNutri = $baseNutri - $this->porcentagem($baseNutri, $perdaLimp);
        $novoCalcNutri = $baseNutri * $aprovNutri / 100;
        return $novoCalcNutri;
    }
    
    private function porcentagem($qtdTotal, $qtdPorcent){
        return $qtdTotal * ($qtdPorcent / 100);
    }
} 